@@ -20,12 +20,19 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 import gc as garbageCollector
 
 #garbageCollector.set_debug(garbageCollector.DEBUG_LEAK)
 
+
+__all__ = [
+	"AwlDebug",
+]
+
+
 class AwlDebug(object):
 	@classmethod
 	def getMemoryLeaks(cls):
