@@ -20,17 +20,33 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
+from awlsim.common.exceptions import *
+
+
+__all__ = [
+	"importModule",
+]
+
 
 def importModule(moduleName):
 	"""Import a module with the name string 'moduleName'.
 	Returns the module object.
 	May raise importError."""
 
-	import awlsim.cython_helper as cython_helper
-	import importlib
+	import awlsim_loader.cython_helper as cython_helper
+	try:
+		import importlib
+	except ImportError as e:
+		importlib = None
 
 	if cython_helper.shouldUseCython(moduleName):
 		moduleName = cython_helper.cythonModuleName(moduleName)
-	return importlib.import_module(moduleName)
+	if importlib:
+		mod = importlib.import_module(moduleName)
+	else:
+		mod = __import__(moduleName)
+
+	return mod
