@@ -2,7 +2,7 @@
 #
 # AWL simulator - utility functions
 #
-# Copyright 2012-2013 Michael Buesch <m@bues.ch>
+# Copyright 2012-2015 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,9 +20,15 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 
+__all__ = [
+	"EnumGen",
+]
+
+
 class MagicEnumerationGenerator(object):
 	"Magic enumeration generator"
 
@@ -68,6 +74,7 @@ class MagicEnumerationGenerator(object):
 
 	def itemAt(self, number):
 		assert(self.__num is not None)
+		assert(number >= self.__num)
 		self.__num = number + 1
 		return number
 
