@@ -2,7 +2,7 @@
 #
 # AWL simulator - datablocks
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,23 +20,36 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
+from awlsim.common.blockinfo import *
+from awlsim.common.enumeration import *
+from awlsim.common.exceptions import *
+
 from awlsim.core.util import *
-from awlsim.core.operators import *
+from awlsim.core.operators import * #+cimport
 from awlsim.core.datatypes import *
-from awlsim.core.datastructure import *
-from awlsim.core.blocks import *
+from awlsim.core.memory import * #+cimport
+from awlsim.core.datastructure import * #+cimport
+from awlsim.core.blocks import * #+cimport
+from awlsim.core.offset import * #+cimport
 
 
-class DB(Block):
-	PERM_READ	= 1 << 0
-	PERM_WRITE	= 1 << 1
+class DB(Block): #+cdef
+	EnumGen.start
+	PERM_READ	= EnumGen.bitmask
+	PERM_WRITE	= EnumGen.bitmask
+	EnumGen.end
 
 	def __init__(self, index, codeBlock=None,
 		     permissions=(PERM_READ|PERM_WRITE)):
 		Block.__init__(self, index)
+		self._PERM_READ = self.PERM_READ
+		self._PERM_WRITE = self.PERM_WRITE
+
 		self.setPermissions(permissions)
+
 		self.codeBlock = codeBlock	# The FB, if this is an instance-DB.
 		if self.codeBlock:
 			# The data structure is declared by the interface.
@@ -47,19 +60,11 @@ class DB(Block):
 
 	def setPermissions(self, newPermissions):
 		self.permissions = newPermissions
-		if self.permissions & self.PERM_READ:
-			self.fetch = self.__fetch
-		else:
-			self.fetch = self.__fetch_noPermission
-		if self.permissions & self.PERM_WRITE:
-			self.store = self.__store
-		else:
-			self.store = self.__store_noPermission
 
 	@property
 	def struct(self):
 		if self.codeBlock:
-			return self.codeBlock.interface.struct
+			return self.codeBlock.interface._struct
 		return self.__struct
 
 	def isInstanceDB(self):
@@ -68,24 +73,41 @@ class DB(Block):
 	def allocate(self):
 		self.structInstance = AwlStructInstance(self.struct)
 
-	def __fetch(self, operator, baseOffset=AwlOffset()):
-		return self.structInstance.dataBytes.fetch(baseOffset + operator.value,
-							   operator.width)
-
-	def __fetch_noPermission(self, operator, baseOffset=None):
+	def fetch(self, operator, baseOffset): #@nocy
+#@cy	cdef object fetch(self, AwlOperator operator, AwlOffset baseOffset):
+		if self.permissions & self._PERM_READ:
+			if baseOffset is None:
+				return self.structInstance.memory.fetch(
+						operator.offset,
+						operator.width)
+			else:
+				return self.structInstance.memory.fetch(
+						baseOffset + operator.offset,
+						operator.width)
 		raise AwlSimError("Fetch from read protected DB %d" % self.index)
 
-	fetch = __fetch
+	def store(self, operator, value, baseOffset): #@nocy
+#@cy	cdef store(self, AwlOperator operator, object value, AwlOffset baseOffset):
+		if self.permissions & self._PERM_WRITE:
+			if baseOffset is None:
+				self.structInstance.memory.store(
+						operator.offset,
+						operator.width,
+						value)
+			else:
+				self.structInstance.memory.store(
+						baseOffset + operator.offset,
+						operator.width,
+						value)
+		else:
+			raise AwlSimError("Store to write protected DB %d" % self.index)
 
-	def __store(self, operator, value, baseOffset=AwlOffset()):
-		self.structInstance.dataBytes.store(baseOffset + operator.value,
-						    operator.width,
-						    value)
-
-	def __store_noPermission(self, operator, value, baseOffset=None):
-		raise AwlSimError("Store to write protected DB %d" % self.index)
-
-	store = __store
+	def getBlockInfo(self):
+		"""Get a BlockInfo instance for this block.
+		"""
+		return BlockInfo(blockType = BlockInfo.TYPE_DB,
+				 blockIndex = self.index,
+				 identHash = self.identHash)
 
 	def __repr__(self):
 		if self.index == 0:
