@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,24 +20,32 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_ASSIGN(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_ASSIGN, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_ASSIGN, rawInsn, **kwargs)
 		self.assertOpCount(1)
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
+#@cy		cdef S7CPU cpu
 
-		s = self.cpu.statusWord
-		if self.cpu.mcrActive and not self.cpu.mcrIsOn():
+		cpu = self.cpu
+		s = cpu.statusWord
+		if cpu.mcrActive and not cpu.mcrIsOn():
 			s.OR, s.STA, s.NER = 0, 0, 0
 		else:
 			s.OR, s.STA, s.NER = 0, s.VKE, 0
-		self.cpu.store(self.ops[0], s.STA, (1,))
+		cpu.store(self.op0, s.STA, self._widths_1)
