@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,67 +20,73 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_AbstractCall(AwlInsn): #+cdef
+
+	__slots__ = ()
+
 	def staticSanityChecks(self):
-		if len(self.ops) == 1:
+		if self.opCount == 1:
 			# "CALL FC/SFC" or
 			# "CALL #MULTIINSTANCE" or
 			# "UC/CC FC/SFC/FB/SFB"
-			blockOper = self.ops[0]
+			blockOper = self.op0
 
-			if blockOper.type == AwlOperator.BLKREF_FC:
+			if blockOper.operType == AwlOperatorTypes.BLKREF_FC:
 				try:
-					codeBlock = self.cpu.fcs[blockOper.value.byteOffset]
+					codeBlock = self.cpu.fcs[blockOper.offset.byteOffset]
 				except KeyError as e:
 					raise AwlSimError("Called FC not found",
-						rawInsn = self.rawInsn)
-			elif blockOper.type == AwlOperator.BLKREF_SFC:
+						rawInsn=self.getRawInsn())
+			elif blockOper.operType == AwlOperatorTypes.BLKREF_SFC:
 				try:
-					codeBlock = self.cpu.sfcs[blockOper.value.byteOffset]
+					codeBlock = self.cpu.sfcs[blockOper.offset.byteOffset]
 				except KeyError as e:
 					raise AwlSimError("SFC %d not implemented, yet" %\
-						blockOper.value.byteOffset,
-						rawInsn = self.rawInsn)
-			elif blockOper.type == AwlOperator.BLKREF_FB:
+						blockOper.offset.byteOffset,
+						rawInsn=self.getRawInsn())
+			elif blockOper.operType == AwlOperatorTypes.BLKREF_FB:
 				if self.insnType == AwlInsn.TYPE_CALL:
 					raise AwlSimError("Missing DB in function "
 						"block call",
-						rawInsn = self.rawInsn)
+						rawInsn=self.getRawInsn())
 				try:
-					codeBlock = self.cpu.fbs[blockOper.value.byteOffset]
+					codeBlock = self.cpu.fbs[blockOper.offset.byteOffset]
 				except KeyError as e:
 					raise AwlSimError("Called FB not found",
-						rawInsn = self.rawInsn)
-			elif blockOper.type == AwlOperator.BLKREF_SFB:
+						rawInsn=self.getRawInsn())
+			elif blockOper.operType == AwlOperatorTypes.BLKREF_SFB:
 				if self.insnType == AwlInsn.TYPE_CALL:
 					raise AwlSimError("Missing DB in system function "
 						"block call",
-						rawInsn = self.rawInsn)
+						rawInsn=self.getRawInsn())
 				try:
-					codeBlock = self.cpu.sfbs[blockOper.value.byteOffset]
+					codeBlock = self.cpu.sfbs[blockOper.offset.byteOffset]
 				except KeyError as e:
 					raise AwlSimError("SFB %d not implemented, yet" %\
-						blockOper.value.byteOffset,
-						rawInsn = self.rawInsn)
-			elif blockOper.type == AwlOperator.INDIRECT:
+						blockOper.offset.byteOffset,
+						rawInsn=self.getRawInsn())
+			elif blockOper.operType == AwlOperatorTypes.INDIRECT:
 				# Indirect call. (like UC FC[MW 0])
 				codeBlock = None
-			elif blockOper.type in (AwlOperator.MULTI_FB, AwlOperator.MULTI_SFB):
+			elif blockOper.operType in (AwlOperatorTypes.MULTI_FB, AwlOperatorTypes.MULTI_SFB):
 				# Multi instance call (like CALL #FOO)
-				if blockOper.type == AwlOperator.MULTI_FB:
-					codeBlock = self.cpu.fbs[blockOper.value.fbNumber]
+				if blockOper.operType == AwlOperatorTypes.MULTI_FB:
+					codeBlock = self.cpu.fbs[blockOper.offset.fbNumber]
 				else:
-					codeBlock = self.cpu.sfbs[blockOper.value.fbNumber]
+					codeBlock = self.cpu.sfbs[blockOper.offset.fbNumber]
 			else:
 				raise AwlSimError("Invalid CALL operand",
-					rawInsn = self.rawInsn)
+					rawInsn=self.getRawInsn())
 
 			if self.insnType == AwlInsn.TYPE_CALL and\
 			   codeBlock and\
@@ -91,76 +97,87 @@ class AwlInsn_AbstractCall(AwlInsn): #+cdef
 					"====  The block interface is:\n%s\n====" %\
 					(len(self.params), codeBlock.interface.interfaceFieldCount,
 					 str(codeBlock.interface)),
-					rawInsn = self.rawInsn)
-		elif len(self.ops) == 2:
+					rawInsn=self.getRawInsn())
+		elif self.opCount == 2:
 			# "CALL FB/SFB"
-			blockOper = self.ops[0]
-			dbOper = self.ops[1]
+			blockOper = self.op0
+			dbOper = self.op1
 
-			if dbOper.type != AwlOperator.BLKREF_DB:
+			if dbOper.operType != AwlOperatorTypes.BLKREF_DB:
 				raise AwlSimError("Second CALL operand is "
 					"not a DB operand.",
-					rawInsn = self.rawInsn)
+					rawInsn=self.getRawInsn())
 			try:
-				db = self.cpu.dbs[dbOper.value.byteOffset]
+				db = self.cpu.dbs[dbOper.offset.byteOffset]
 			except KeyError as e:
 				raise AwlSimError("DB used in FB call not found",
-					rawInsn = self.rawInsn)
+					rawInsn=self.getRawInsn())
 			if not db.isInstanceDB():
 				raise AwlSimError("DB %d is not an instance DB" %\
-					dbOper.value.byteOffset,
-					rawInsn = self.rawInsn)
+					dbOper.offset.byteOffset,
+					rawInsn=self.getRawInsn())
 
-			if blockOper.type == AwlOperator.BLKREF_FB:
+			if blockOper.operType == AwlOperatorTypes.BLKREF_FB:
 				try:
-					fb = self.cpu.fbs[blockOper.value.byteOffset]
+					fb = self.cpu.fbs[blockOper.offset.byteOffset]
 				except KeyError as e:
 					raise AwlSimError("Called FB not found",
-						rawInsn = self.rawInsn)
+						rawInsn=self.getRawInsn())
 				# TODO check if this is an FB-DB
 				pass#TODO
-			elif blockOper.type == AwlOperator.BLKREF_SFB:
+			elif blockOper.operType == AwlOperatorTypes.BLKREF_SFB:
 				try:
-					fb = self.cpu.sfbs[blockOper.value.byteOffset]
+					fb = self.cpu.sfbs[blockOper.offset.byteOffset]
 				except KeyError as e:
 					raise AwlSimError("SFB %d not implemented, yet" %\
-						blockOper.value.byteOffset,
-						rawInsn = self.rawInsn)
+						blockOper.offset.byteOffset,
+						rawInsn=self.getRawInsn())
 				# TODO check if this is an SFB-DB
 				pass#TODO
-			elif blockOper.type == AwlOperator.BLKREF_FC or\
-			     blockOper.type == AwlOperator.BLKREF_SFC:
+			elif blockOper.operType == AwlOperatorTypes.BLKREF_FC or\
+			     blockOper.operType == AwlOperatorTypes.BLKREF_SFC:
 				raise AwlSimError("Calling function, but "
 					"a DB was specified.",
-					rawInsn = self.rawInsn)
+					rawInsn=self.getRawInsn())
 			else:
 				raise AwlSimError("Invalid CALL operand",
-					rawInsn = self.rawInsn)
+					rawInsn=self.getRawInsn())
 
 			if db.codeBlock.index != fb.index:
 				raise AwlSimError("DB %d is not an instance DB for FB %d" %\
-					(dbOper.value.byteOffset,
-					 blockOper.value.byteOffset),
-					rawInsn = self.rawInsn)
+					(dbOper.offset.byteOffset,
+					 blockOper.offset.byteOffset),
+					rawInsn=self.getRawInsn())
 		else:
 			assert(0)
 		# Check parameter assignments
 		for param in self.params:
 			if param.isOutbound:
-				if param.rvalueOp.isImmediate():
+				if ((blockOper.operType == AwlOperatorTypes.BLKREF_FB or\
+				     blockOper.operType == AwlOperatorTypes.BLKREF_SFB) and\
+				    param.rvalueOp.isImmediate()) or\
+				   ((blockOper.operType == AwlOperatorTypes.BLKREF_FC or\
+				     blockOper.operType == AwlOperatorTypes.BLKREF_SFC) and\
+				    param.rvalueOp.isImmediate() and\
+				    param.rvalueOp.operType != AwlOperatorTypes.IMM_PTR):
 					raise AwlSimError("Immediate value assignment '%s' "
 						"to OUTPUT or IN_OUT parameter '%s' is "
 						"not allowed." %\
 						(str(param.rvalueOp),
 						 param.lvalueName),
-						rawInsn = self.rawInsn)
+						rawInsn=self.getRawInsn())
 
 class AwlInsn_CALL(AwlInsn_AbstractCall): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn_AbstractCall.__init__(self, cpu, AwlInsn.TYPE_CALL, rawInsn)
-		self.assertOpCount((1,2))
 
-		if len(self.ops) == 1:			#@nocy
+	__slots__ = (
+		"run",
+	)
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn_AbstractCall.__init__(self, cpu, AwlInsn.TYPE_CALL, rawInsn, **kwargs)
+		self.assertOpCount((1, 2))
+
+		if self.opCount == 1:			#@nocy
 			self.run = self.__run_CALL_FC	#@nocy
 		else:					#@nocy
 			self.run = self.__run_CALL_FB	#@nocy
@@ -168,44 +185,56 @@ class AwlInsn_CALL(AwlInsn_AbstractCall): #+cdef
 	def __run_CALL_FC(self): #+cdef
 #@cy		cdef S7StatusWord s
 
-		self.cpu.run_CALL(self.ops[0], None, self.params, False)
+		self.cpu.run_CALL(self.op0, None, self.params, False)
 		s = self.cpu.statusWord
 		s.OS, s.OR, s.STA, s.NER = 0, 0, 1, 0
 
 	def __run_CALL_FB(self): #+cdef
 #@cy		cdef S7StatusWord s
 
-		self.cpu.run_CALL(self.ops[0], self.ops[1], self.params, False)
+		self.cpu.run_CALL(self.op0, self.op1, self.params, False)
 		s = self.cpu.statusWord
 		s.OS, s.OR, s.STA, s.NER = 0, 0, 1, 0
 
-#@cy	def run(self):
-#@cy		if len(self.ops) == 1:
+#@cy	cdef run(self):
+#@cy		if self.opCount == 1:
 #@cy			self.__run_CALL_FC()
 #@cy		else:
 #@cy			self.__run_CALL_FB()
 
 class AwlInsn_CC(AwlInsn_AbstractCall): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn_AbstractCall.__init__(self, cpu, AwlInsn.TYPE_CC, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn_AbstractCall.__init__(self, cpu, AwlInsn.TYPE_CC, rawInsn, **kwargs)
 		self.assertOpCount(1)
 
-	def run(self):
+	def staticSanityChecks(self):
+		self._warnDeprecated("Please use CALL instead.")
+
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
 
 		s = self.cpu.statusWord
 		if s.VKE:
-			self.cpu.run_CALL(self.ops[0], None, (), True)
+			self.cpu.run_CALL(self.op0, None, (), True)
 		s.OS, s.OR, s.STA, s.VKE, s.NER = 0, 0, 1, 1, 0
 
 class AwlInsn_UC(AwlInsn_AbstractCall): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn_AbstractCall.__init__(self, cpu, AwlInsn.TYPE_UC, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn_AbstractCall.__init__(self, cpu, AwlInsn.TYPE_UC, rawInsn, **kwargs)
 		self.assertOpCount(1)
 
-	def run(self):
+	def staticSanityChecks(self):
+		self._warnDeprecated("Please use CALL instead.")
+
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
 
-		self.cpu.run_CALL(self.ops[0], None, (), True)
+		self.cpu.run_CALL(self.op0, None, (), True)
 		s = self.cpu.statusWord
 		s.OS, s.OR, s.STA, s.NER = 0, 0, 1, 0
