@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,22 +20,30 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+from awlsim.common.datatypehelpers import * #+cimport
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 import math
 
 
 class AwlInsn_COS(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_COS, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_COS, rawInsn, **kwargs)
 		self.assertOpCount(0)
 
-	def run(self):
-#@cy		cdef S7StatusWord s
+	def run(self): #+cdef
+#@cy		cdef double accu1
+#@cy		cdef double extremum
 
 		accu1 = math.cos(self.cpu.accu1.getPyFloat())
 		for extremum in (-1.0, 0.0, 1.0):
