@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,28 +20,37 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_INCAR1(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_INCAR1, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_INCAR1, rawInsn, **kwargs)
 		self.assertOpCount((0, 1))
-		if self.ops:
-			self.ops[0].assertType(AwlOperator.IMM_PTR)
+		if self.opCount:
+			self.op0.assertType(AwlOperatorTypes.IMM_PTR, widths=self._widths_32)
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
+#@cy		cdef S7CPU cpu
+#@cy		cdef uint32_t ar
 
-		ar = self.cpu.ar1.get()
-		if self.ops:
+		cpu = self.cpu
+		ar = cpu.ar1.get()
+		if self.opCount:
 			ar = (ar & 0xFF000000) |\
-			     (((ar & 0x00FFFFFF) + self.ops[0].value) & 0x00FFFFFF)
+			     (((ar & 0x00FFFFFF) + self.op0.pointer.toPointerValue()) & 0x00FFFFFF)
 		else:
 			ar = (ar & 0xFF000000) |\
-			     (((ar & 0x00FFFFFF) + self.cpu.accu1.getSignedWord()) & 0x00FFFFFF)
-		self.cpu.ar1.set(ar)
+			     (((ar & 0x00FFFFFF) + cpu.accu1.getSignedWord()) & 0x00FFFFFF)
+		cpu.ar1.set(ar)
