@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,23 +20,32 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_NEGD(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_NEGD, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_NEGD, rawInsn, **kwargs)
 		self.assertOpCount(0)
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
+#@cy		cdef int64_t value
+#@cy		cdef int32_t accu1
 
 		s = self.cpu.statusWord
-		value = -(self.cpu.accu1.getSignedDWord())
+		value = self.cpu.accu1.getSignedDWord()
+		value = -value
 		self.cpu.accu1.setDWord(value)
 		accu1 = self.cpu.accu1.getSignedDWord()
 		if accu1 == 0:
@@ -45,5 +54,6 @@ class AwlInsn_NEGD(AwlInsn): #+cdef
 			s.A1, s.A0, s.OV = 0, 1, 0
 		else:
 			s.A1, s.A0, s.OV = 1, 0, 0
-		if value > 0x7FFFFFFF or value < -2147483648:
+		if value > 0x7FFFFFFF or value < -2147483648: #@nocy
+#@cy		if value > 0x7FFFFFFFLL or value < -2147483648LL:
 			s.OV, s.OS = 1, 1
