@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,27 +20,36 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_S(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_S, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_S, rawInsn, **kwargs)
 		self.assertOpCount(1)
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
+#@cy		cdef S7CPU cpu
+#@cy		cdef AwlOperator oper
 
-		s, oper = self.cpu.statusWord,\
-			self.ops[0].resolve(True)
-		if oper.type == AwlOperator.MEM_Z:
-			self.cpu.getCounter(oper.value.byteOffset).set(s.VKE)
+		cpu = self.cpu
+		s = cpu.statusWord
+		oper = self.op0.resolve(True)
+		if oper.operType == AwlOperatorTypes.MEM_Z:
+			cpu.getCounter(oper.offset.byteOffset).set(s.VKE)
 			s.OR, s.NER = 0, 0
 		else:
-			if s.VKE and (not self.cpu.mcrActive or self.cpu.mcrIsOn()):
-				self.cpu.store(oper, 1, (1,))
+			if s.VKE and (not cpu.mcrActive or cpu.mcrIsOn()):
+				cpu.store(oper, 1, self._widths_1)
 			s.OR, s.STA, s.NER = 0, s.VKE, 0
