@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,29 +20,36 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+from awlsim.common.datatypehelpers import * #+cimport
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 import math
 
 
 class AwlInsn_SQRT(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_SQRT, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_SQRT, rawInsn, **kwargs)
 		self.assertOpCount(0)
 
-	def run(self):
-#@cy		cdef S7StatusWord s
+	def run(self): #+cdef
+#@cy		cdef double accu1
 
 		accu1 = self.cpu.accu1.getPyFloat()
 		try:
 			accu1 = math.sqrt(accu1)
 		except ValueError:
-			self.cpu.accu1.setDWord(pNaNDWord)
-			accu1 = nNaNFloat
+			self.cpu.accu1.setDWord(floatConst.pNaNDWord)
+			accu1 = floatConst.nNaNFloat
 		else:
 			self.cpu.accu1.setPyFloat(accu1)
 		self.cpu.statusWord.setForFloatingPoint(accu1)
