@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,23 +20,29 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_TAR1(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_TAR1, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_TAR1, rawInsn, **kwargs)
 		self.assertOpCount((0, 1))
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
 
-		if self.ops:
-			self.cpu.store(self.ops[0], self.cpu.ar1.get(), (32,))
+		if self.opCount:
+			self.cpu.store(self.op0, self.cpu.ar1.get(), self._widths_32)
 		else:
 			self.cpu.accu2.copyFrom(self.cpu.accu1)
 			self.cpu.accu1.copyFrom(self.cpu.ar1)
