@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,28 +20,35 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_TRUNC(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_TRUNC, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_TRUNC, rawInsn, **kwargs)
 		self.assertOpCount(0)
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
 
 		s = self.cpu.statusWord
 		accu1 = self.cpu.accu1.getPyFloat()
 		try:
 			accu1 = int(accu1)
-			if accu1 > 2147483647 or accu1 < -2147483648:
+			if accu1 > 2147483647 or accu1 < -2147483648: #@nocy
+#@cy			if accu1 > 2147483647LL or accu1 < -2147483648LL:
 				raise ValueError
-		except ValueError:
+		except (ValueError, OverflowError) as e:
 			s.OV, s.OS = 1, 1
 			return
 		self.cpu.accu1.setDWord(accu1)
