@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,27 +20,33 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_UD(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_UD, rawInsn)
-		self.assertOpCount((0, 1))
-		if self.ops:
-			self.ops[0].assertType(AwlOperator.IMM)
 
-	def run(self):
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_UD, rawInsn, **kwargs)
+		self.assertOpCount((0, 1))
+		if self.opCount:
+			self.op0.assertType(AwlOperatorTypes.IMM)
+
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
 
 		s = self.cpu.statusWord
 		accu1 = self.cpu.accu1.getDWord()
-		if self.ops:
-			accu2 = self.ops[0].value
+		if self.opCount:
+			accu2 = self.op0.immediate
 		else:
 			accu2 = self.cpu.accu2.getDWord()
 		accu1 &= accu2
