@@ -2,7 +2,7 @@
 #
 # AWL simulator - instructions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,24 +20,33 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.instructions.main import * #@nocy
-from awlsim.core.operators import *
-#from awlsim.core.instructions.main cimport * #@cy
+from awlsim.common.exceptions import *
+
+from awlsim.core.instructions.main import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
 
 
 class AwlInsn_UN(AwlInsn): #+cdef
-	def __init__(self, cpu, rawInsn):
-		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_UN, rawInsn)
+
+	__slots__ = ()
+
+	def __init__(self, cpu, rawInsn=None, **kwargs):
+		AwlInsn.__init__(self, cpu, AwlInsn.TYPE_UN, rawInsn, **kwargs)
 		self.assertOpCount(1)
 
-	def run(self):
+	def run(self): #+cdef
 #@cy		cdef S7StatusWord s
+#@cy		cdef _Bool NER
+#@cy		cdef _Bool STA
+#@cy		cdef _Bool newOR
 
-		s, STA = self.cpu.statusWord,\
-			self.cpu.fetch(self.ops[0], (1,))
-		if s.NER:
-			s.STA, s.VKE = STA, ((s.VKE & (STA ^ 1)) | s.OR)
-		else:
-			s.STA, s.VKE, s.NER = STA, (STA ^ 1), 1
+		s = self.cpu.statusWord
+		STA = self.cpu.fetch(self.op0, self._widths_1)
+		NER = s.NER
+		newOR = s.OR & NER
+		s.VKE = ((s.VKE | (NER ^ 1)) & (STA ^ 1)) | newOR
+		s.OR, s.STA, s.NER = newOR, STA, 1
