@@ -20,11 +20,15 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-import math
+from awlsim.common.exceptions import *
 
 from awlsim.core.util import *
+from awlsim.core.cpu import * #+cimport
+
+import math
 
 
 class OBTempPresets(object):
@@ -60,6 +64,8 @@ class OB1TempPresets(OBTempPresets):
 		OBTempPresets.__init__(self, 1, cpu)
 
 	def generate(self, localdata):
+#@cy		cdef S7CPU cpu
+
 		cpu, ceil = self.cpu, math.ceil
 		try:
 			avgMs = min(0x7FFF, int(ceil(cpu.avgCycleTime * 1000)))
