@@ -2,7 +2,7 @@
 #
 # AWL simulator - status word
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,19 +20,20 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-#from awlsim.core.dynattrs cimport * #@cy
 #from awlsim.core.statusword cimport * #@cy
 
-from awlsim.common.cpuspecs import *
-from awlsim.common.datatypehelpers import *
+from awlsim.common.cpuspecs import * #+cimport
+from awlsim.common.cpuconfig import *
+from awlsim.common.datatypehelpers import * #+cimport
+from awlsim.common.exceptions import *
 
-from awlsim.core.dynattrs import * #@nocy
 from awlsim.core.util import *
 
 
-class S7StatusWord(DynAttrs): #+cdef
+class S7StatusWord(object): #+cdef
 	"""STEP 7 status word
 	The instance of this class holds the following nine
 	attributes. One for each STW bit:
@@ -65,26 +66,53 @@ class S7StatusWord(DynAttrs): #+cdef
 	}
 	__german2english = pivotDict(__english2german)
 
+	# Number of status bits in the status word.
 	NR_BITS = 9
 
-	# Dynamic attributes
-	dynAttrs = {
-		"NER"	: 0,
-		"VKE"	: 0,
-		"STA"	: 0,
-		"OR"	: 0,
-		"OS"	: 0,
-		"OV"	: 0,
-		"A0"	: 0,
-		"A1"	: 0,
-		"BIE"	: 0,
-	}
+	# Default values for status bits.
+	NER	= 0 #@nocy
+	VKE	= 0 #@nocy
+	STA	= 0 #@nocy
+	OR	= 0 #@nocy
+	OS	= 0 #@nocy
+	OV	= 0 #@nocy
+	A0	= 0 #@nocy
+	A1	= 0 #@nocy
+	BIE	= 0 #@nocy
+
+#@cy	def __cinit__(self):
+#@cy		self.NER = 0
+#@cy		self.VKE = 0
+#@cy		self.STA = 0
+#@cy		self.OR = 0
+#@cy		self.OS = 0
+#@cy		self.OV = 0
+#@cy		self.A0 = 0
+#@cy		self.A1 = 0
+#@cy		self.BIE = 0
+
+	def __eq__(self, other): #@nocy
+#@cy	cdef __eq(self, object other):
+		return (self is other) or (\
+			isinstance(other, S7StatusWord) and\
+			self.getWord() == other.getWord()\
+		)
+
+	def __ne__(self, other):		#@nocy
+		return not self.__eq__(other)	#@nocy
+
+#@cy	def __richcmp__(self, object other, int op):
+#@cy		if op == 2: # __eq__
+#@cy			return self.__eq(other)
+#@cy		elif op == 3: # __ne__
+#@cy			return not self.__eq(other)
+#@cy		return False
 
 	@classmethod
 	def getBitnrByName(cls, name, mnemonics):
-		assert(mnemonics != S7CPUSpecs.MNEMONICS_AUTO)
+		assert(mnemonics != S7CPUConfig.MNEMONICS_AUTO)
 		try:
-			if mnemonics == S7CPUSpecs.MNEMONICS_EN:
+			if mnemonics == S7CPUConfig.MNEMONICS_EN:
 				name = cls.__english2german[name]
 			return cls.name2nr_german[name]
 		except KeyError as e:
@@ -130,27 +158,49 @@ class S7StatusWord(DynAttrs): #+cdef
 		__getBIE,
 	)
 
-	def getByBitNumber(self, bitNumber):
-		try:
-			return self.__bitnr2getter[bitNumber](self)
-		except IndexError as e:
-			raise AwlSimError("Status word bit fetch '%d' "
-				"out of range" % bitNumber)
+	def getByBitNumber(self, bitNumber):					#@nocy
+		try:								#@nocy
+			return self.__bitnr2getter[bitNumber](self)		#@nocy
+		except IndexError as e:						#@nocy
+			raise AwlSimError("Status word bit fetch '%d' "		#@nocy
+				"out of range" % bitNumber)			#@nocy
+
+#@cy	cdef _Bool getByBitNumber(self, uint8_t bitNumber):
+#@cy		if bitNumber == 0:
+#@cy			return self.NER
+#@cy		elif bitNumber == 1:
+#@cy			return self.VKE
+#@cy		elif bitNumber == 2:
+#@cy			return self.STA
+#@cy		elif bitNumber == 3:
+#@cy			return self.OR
+#@cy		elif bitNumber == 4:
+#@cy			return self.OS
+#@cy		elif bitNumber == 5:
+#@cy			return self.OV
+#@cy		elif bitNumber == 6:
+#@cy			return self.A0
+#@cy		elif bitNumber == 7:
+#@cy			return self.A1
+#@cy		elif bitNumber == 8:
+#@cy			return self.BIE
+#@cy		raise AwlSimError("Status word bit fetch '%d' "
+#@cy			"out of range" % bitNumber)
 
 	def reset(self): #@nocy
-#@cy	cpdef reset(self):
+#@cy	cdef void reset(self):
 		self.NER = self.VKE = self.STA =\
 		self.OR = self.OS = self.OV =\
 		self.A0 = self.A1 = self.BIE = 0
 
 	def getWord(self): #@nocy
-#@cy	cpdef uint16_t getWord(self):
+#@cy	cdef uint16_t getWord(self):
 		return self.NER | (self.VKE << 1) | (self.STA << 2) |\
 		       (self.OR << 3) | (self.OS << 4) | (self.OV << 5) |\
 		       (self.A0 << 6) | (self.A1 << 7) | (self.BIE << 8)
 
 	def setWord(self, word): #@nocy
-#@cy	cpdef setWord(self, uint16_t word):
+#@cy	cdef void setWord(self, uint16_t word):
 		self.NER = word & 1
 		self.VKE = (word >> 1) & 1
 		self.STA = (word >> 2) & 1
@@ -161,8 +211,7 @@ class S7StatusWord(DynAttrs): #+cdef
 		self.A1 = (word >> 7) & 1
 		self.BIE = (word >> 8) & 1
 
-	def dup(self): #@nocy
-#@cy	cpdef dup(self):
+	def dup(self): #+cdef
 		new = S7StatusWord()
 		new.NER = self.NER
 		new.VKE = self.VKE
@@ -176,7 +225,7 @@ class S7StatusWord(DynAttrs): #+cdef
 		return new
 
 	def setForFloatingPoint(self, pyFloat): #@nocy
-#@cy	cpdef setForFloatingPoint(self, pyFloat):
+#@cy	cdef void setForFloatingPoint(self, double pyFloat):
 		dword = pyFloatToDWord(pyFloat)
 		dwordNoSign = dword & 0x7FFFFFFF
 		if isDenormalPyFloat(pyFloat) or\
@@ -204,15 +253,15 @@ class S7StatusWord(DynAttrs): #+cdef
 			self.A1, self.A0, self.OV = 1, 0, 0
 
 	def getString(self, mnemonics):
-		if mnemonics == S7CPUSpecs.MNEMONICS_AUTO:
-			mnemonics = S7CPUSpecs.MNEMONICS_EN
+		if mnemonics == S7CPUConfig.MNEMONICS_AUTO:
+			mnemonics = S7CPUConfig.MNEMONICS_EN
 		ret = []
 		for i in range(self.NR_BITS - 1, -1, -1):
 			name = self.nr2name_german[i]
-			if mnemonics == S7CPUSpecs.MNEMONICS_EN:
+			if mnemonics == S7CPUConfig.MNEMONICS_EN:
 				name = self.__german2english[name]
 			ret.append("%s:%d" % (name, self.getByBitNumber(i)))
 		return '  '.join(ret)
 
 	def __repr__(self):
-		return self.getString(S7CPUSpecs.MNEMONICS_DE)
+		return self.getString(S7CPUConfig.MNEMONICS_DE)
