@@ -2,7 +2,7 @@
 #
 # AWL simulator - symbol table parser
 #
-# Copyright 2014 Michael Buesch <m@bues.ch>
+# Copyright 2014-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,16 +20,29 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.common.cpuspecs import *
-from awlsim.core.datatypes import *
-from awlsim.core.optrans import *
+from awlsim.common.exceptions import *
+from awlsim.common.cpuconfig import *
+
+from awlsim.core.memory import * #+cimport
 from awlsim.core.util import *
 
+from awlsim.awlcompiler.optrans import *
+
 import csv
 
 
+__all__ = [
+	"Symbol",
+	"SymbolTable",
+	"SymTabParser",
+	"SymTabParser_ASC",
+	"SymTabParser_CSV",
+]
+
+
 class AwlsimSymTabCSVDialect(csv.Dialect):
 	delimiter = str(';')
 	quotechar = str('"')
@@ -37,38 +50,29 @@ class AwlsimSymTabCSVDialect(csv.Dialect):
 	skipinitialspace = True
 	lineterminator = str('\r\n')
 	quoting = csv.QUOTE_MINIMAL
+	escapechar = None
 csv.register_dialect("awlsim_symtab", AwlsimSymTabCSVDialect)
 
 class Symbol(object):
-	"""One symbol."""
+	"""One symbol.
+	"""
 
-	def __init__(self, name=None, operator=None, type=None, comment="",
-		     mnemonics=S7CPUSpecs.MNEMONICS_AUTO,
+	def __init__(self, name="", operator=None, type=None, comment="",
+		     mnemonics=S7CPUConfig.MNEMONICS_AUTO,
 		     lineNr=None, symTab=None):
+		self.setSymTab(symTab)
 		self.setName(name)		# The symbol name string
 		self.setOperator(operator)	# The symbol address (AwlOperator)
 		self.setType(type)		# The symbol type (AwlDataType)
 		self.setComment(comment)	# The comment string
 		self.setMnemonics(mnemonics)
 		self.setLineNr(lineNr)
-		self.setSymTab(symTab)
 
 	def isValid(self):
-		return self.name and self.operator and self.type and self.isUnique()
-
-	def isUnique(self):
-		if not self.symTab:
-			return True
-		for sym in self.symTab.findByName(self.name):
-			if sym is not self:
-				return False
-		return True
+		return self.name and self.operator and self.type
 
 	def validate(self):
 		name = self.name if self.name else "<no name>"
-		if not self.isUnique():
-			raise AwlSimError("Symbol '%s' is not unique in the symbol table. "
-				"Cannot generate symbol information." % name)
 		if not self.isValid():
 			raise AwlSimError("Symbol '%s' is incomplete. "
 				"Cannot generate symbol information." % name)
@@ -95,19 +99,15 @@ class Symbol(object):
 		if not newOperatorString.strip():
 			self.setOperator(None)
 			return
-		for m in (S7CPUSpecs.MNEMONICS_EN, S7CPUSpecs.MNEMONICS_DE):
-			if self.mnemonics != S7CPUSpecs.MNEMONICS_AUTO and\
+		for m in (S7CPUConfig.MNEMONICS_EN, S7CPUConfig.MNEMONICS_DE):
+			if self.mnemonics != S7CPUConfig.MNEMONICS_AUTO and\
 			   self.mnemonics != m:
 				continue
-			try:
-				opTrans = AwlOpTranslator(insn = None,
-							  mnemonics = m)
-				opDesc = opTrans.translateOp(rawInsn = None,
-							     rawOps = newOperatorString.split())
+			with contextlib.suppress(AwlSimError):
+				opTrans = AwlOpTranslator(mnemonics=m)
+				opDesc = opTrans.translateFromString(newOperatorString)
 				self.setOperator(opDesc.operator)
 				return
-			except AwlSimError as e:
-				pass
 		raise AwlSimError("Symbol table parser: Can't parse symbol "
 			"address '%s' in line %s" %\
 			(newOperatorString, self.getLineNrString()))
@@ -129,6 +129,7 @@ class Symbol(object):
 			self.setType(None)
 			return
 		try:
+			from awlsim.core.datatypes import AwlDataType
 			awlType = AwlDataType.makeByName(newTypeString.split())
 		except AwlSimError as e:
 			raise AwlSimError("Symbol table parser: Can't parse symbol "
@@ -166,29 +167,28 @@ class Symbol(object):
 	def nameIsEqual(self, otherName):
 		if self.name is None or otherName is None:
 			return False
-		return self.name.upper() == otherName.upper()
+		return self.name.lower() == otherName.lower()
 
 	def __csvRecord(self, value):
-		value = str(value).encode(SymTabParser_CSV.ENCODING)
-		value = value.replace(b'"', b'""')
-		if b';' in value or\
-		   b'"' in value or\
-		   b'\r' in value or\
-		   b'\n' in value:
-			value = b'"' + value + b'"'
+		value = str(value)
+		value = value.replace('"', '""')
+		if ';' in value or\
+		   '"' in value or\
+		   '\r' in value or\
+		   '\n' in value:
+			value = '"' + value + '"'
 		return value
 
 	def toCSV(self):
 		# Returns compact CSV of this symbol.
-		# Return type is bytes.
 		self.validate()
 		try:
 			name = self.__csvRecord(self.name)
 			operator = self.__csvRecord(self.operator)
 			type = self.__csvRecord(self.type)
 			comment = self.__csvRecord(self.comment)
-			return b''.join((name, b';', operator, b';',
-					 type, b';', comment, b'\r\n'))
+			return ''.join((name, ';', operator, ';',
+					 type, ';', comment, '\r\n'))
 		except UnicodeError as e:
 			raise AwlSimError("Unicode error while trying to generate "
 				"symbol CSV dump.")
@@ -196,119 +196,211 @@ class Symbol(object):
 	def toReadableCSV(self):
 		# Returns human readable, but also machine processable
 		# CSV of this symbol.
-		# Return type is bytes.
 		self.validate()
 		try:
 			name = self.__csvRecord(self.name)
 			operator = self.__csvRecord(self.operator)
 			type = self.__csvRecord(self.type)
 			comment = self.__csvRecord(self.comment)
-			namePadding = b" " * (24 - len(name)) + b" "
-			operatorPadding = b" " * (11 - len(operator)) + b" "
+			namePadding = " " * (24 - len(name)) + " "
+			operatorPadding = " " * (11 - len(operator)) + " "
 			if comment:
-				typePadding = b" " * (9 - len(type)) + b" "
+				typePadding = " " * (9 - len(type)) + " "
 			else:
-				typePadding = b""
-			return b''.join((name, b';', namePadding,
-					 operator, b';', operatorPadding,
-					 type, b';', typePadding,
-					 comment, b'\r\n'))
+				typePadding = ""
+			return ''.join((name, ';', namePadding,
+					operator, ';', operatorPadding,
+					type, ';', typePadding,
+					comment, '\r\n'))
 		except UnicodeError as e:
 			raise AwlSimError("Unicode error while trying to generate "
 				"symbol CSV dump.")
 
-	def toASC(self):
+	def toASC(self, stripWhitespace=False):
 		# Returns ASC format of this symbol.
-		# Return type is bytes.
 		self.validate()
 		try:
-			name = str(self.name).encode(SymTabParser_ASC.ENCODING)
-			operator = str(self.operator).encode(SymTabParser_ASC.ENCODING)
-			type = str(self.type).encode(SymTabParser_ASC.ENCODING)
-			comment = str(self.comment).encode(SymTabParser_ASC.ENCODING)
-			name += b" " * (24 - len(name))
-			operator += b" " * (11 - len(operator))
-			type += b" " * (9 - len(type))
-			comment += b" " * (80 - len(comment))
-			return b''.join((b'126,', name, operator,
-					 b' ', type, b' ', comment, b'\r\n'))
+			name = str(self.name)
+			operator = str(self.operator)
+			type = str(self.type)
+			comment = str(self.comment)
+			name += " " * (24 - len(name))
+			operator += " " * (11 - len(operator))
+			type += " " * (9 - len(type))
+			comment += " " * (80 - len(comment))
+			line = ''.join(('126,', name, operator,
+					' ', type, ' ', comment))
+			if stripWhitespace:
+				# Strip the right hand white space.
+				# This does not produce compliant ASC format.
+				# But the Awlsim parser supports this as
+				# an editing convenience.
+				# See SymTabParser_ASC.
+				# This is an Awlsim extension.
+				line = line.rstrip()
+				minLen = SymTabParser_ASC.LEN_LEN +\
+					 SymTabParser_ASC.LEN_NAME +\
+					 SymTabParser_ASC.LEN_ADDR + 1
+				if len(line) < minLen:
+					line += " " * (minLen - len(line))
+			line += '\r\n'
 		except UnicodeError as e:
 			raise AwlSimError("Unicode error while trying to generate "
 				"symbol ASC dump.")
+		return line
 
 	def __repr__(self):
-		return self.toReadableCSV().decode(SymTabParser_CSV.ENCODING)
+		return self.toReadableCSV()
 
 class SymbolTable(object):
-	"""Parsed symbol table."""
+	"""Parsed symbol table.
+	"""
 
 	def __init__(self):
 		self.clear()
 
 	def clear(self):
-		self.symbols = []
-
-	def add(self, symbol):
-		if any(self.findByName(symbol.name)):
-			raise AwlSimError("Multiple definitions of "
-				"symbol '%s'" % symbol.name)
-		self.symbols.append(symbol)
-		symbol.setSymTab(self)
-
-	def findByName(self, name):
-		for symbol in self.symbols:
-			if symbol.nameIsEqual(name):
-				yield symbol
-
-	def findOneByName(self, name):
-		foundSyms = tuple(self.findByName(name))
-		if not foundSyms:
-			return None
-		if len(foundSyms) != 1:
-			raise AwlSimError("Multiple definitions of "
-				"symbol '%s'" % symbol.name)
-		return foundSyms[0]
-
-	def merge(self, other):
-		"""Merge 'other' into 'self'"""
-		for symbol in other.symbols:
-			self.add(symbol)
-
+		self.__symbolsList = []
+ 
 	def toCSV(self):
-		return b"".join(s.toCSV() for s in self.symbols)
+		return "".join(s.toCSV()\
+			       for s in self.__symbolsList)
 
 	def toReadableCSV(self):
-		return b"".join(s.toReadableCSV() for s in self.symbols)
+		return "".join(s.toReadableCSV()\
+			       for s in self.__symbolsList)
 
-	def toASC(self):
-		return b"".join(s.toASC() for s in self.symbols)
+	def toASC(self, stripWhitespace=False):
+		return "".join(s.toASC(stripWhitespace=stripWhitespace)\
+			       for s in self.__symbolsList)
+
+	def toSource(self, source):
+		"""Convert the symbol table to bytes for use in the SymTabSource().
+		The source bytes will be written into source.sourceBytes.
+		Raises AwlSimError if something goes wrong.
+		"""
+		try:
+			stripWs = not source.isFileBacked()
+			dataString = self.toASC(stripWhitespace=stripWs)
+			data = dataString.encode(source.ENCODING)
+			source.sourceBytes = data
+		except UnicodeError as e:
+			raise AwlSimError("Failed to encode symbol "
+				"table characters.")
 
 	def __repr__(self):
-		return self.toReadableCSV().decode(SymTabParser_CSV.ENCODING)
+		return self.toReadableCSV()
+
+	def __len__(self):
+		return len(self.__symbolsList)
+
+	def __iter__(self):
+		for symbol in self.__symbolsList:
+			yield symbol
+
+	def __reversed__(self):
+		for symbol in reversed(self.__symbolsList):
+			yield symbol
+
+	def __getitem__(self, index):
+		return self.__symbolsList[index]
+
+	def __setitem__(self, index, symbol):
+		self.pop(index)
+		self.insert(index, symbol)
+
+	def __delitem__(self, index):
+		self.pop(index)
+
+	def __contains__(self, value):
+		if value is None:
+			return False
+		elif isString(value):
+			index, symbol = self.__findByName(value)
+			return symbol is not None
+		elif isinstance(value, Symbol):
+			name = value.getName()
+			if name is None:
+				return False
+			index, symbol = self.__findByName(name)
+			return symbol is not None
+		raise TypeError
+
+	def pop(self, index):
+		"""Get symbol by index and remove it from the table."""
+		symbol = self.__symbolsList.pop(index)
+		symbol.setSymTab(None)
+		return symbol
+
+	def insert(self, index, symbol):
+		"""Insert a symbol before index."""
+		if symbol in self:
+			raise AwlSimError("Multiple definitions of "
+				"symbol '%s'" % symbol.getName())
+		self.__symbolsList.insert(index, symbol)
+		symbol.setSymTab(self)
+
+	def add(self, symbol, overrideExisting = False):
+		if symbol in self:
+			if overrideExisting:
+				i = self.findIndexByName(symbol.getName())
+				assert(i is not None)
+				self[i] = symbol
+			else:
+				raise AwlSimError("Multiple definitions of "
+					"symbol '%s'" % symbol.getName())
+		self.__symbolsList.append(symbol)
+		symbol.setSymTab(self)
+
+	def __findByName(self, name):
+		name = name.lower()
+		for i, symbol in enumerate(self.__symbolsList):
+			if name == symbol.getName().lower():
+				return i, symbol
+		return None, None
+
+	def findByName(self, name):
+		index, symbol = self.__findByName(name)
+		return symbol
+
+	def findIndexByName(self, name):
+		index, symbol = self.__findByName(name)
+		return index
+
+	def getByDataType(self, dataType):
+		"""Get all symbols with the given AwlDataType.
+		Returns a generator.
+		"""
+		return (symbol for symbol in self.__symbolsList\
+			if symbol.getType() == dataType)
+
+	def merge(self, other, overrideExisting = False):
+		"""Merge 'other' into 'self'"""
+		for symbol in other.__symbolsList:
+			self.add(symbol, overrideExisting)
 
 class SymTabParser(object):
-	"""Abstract symbol table parser."""
-
-	ENCODING	= "latin_1"
+	"""Abstract symbol table parser.
+	"""
 
 	implementations = []
 
 	@classmethod
 	def parseSource(cls, source,
 			autodetectFormat=True,
-			mnemonics=S7CPUSpecs.MNEMONICS_AUTO):
-		return cls.parseData(source.sourceBytes, autodetectFormat, mnemonics)
+			mnemonics=S7CPUConfig.MNEMONICS_AUTO):
+		return cls.parseText(source.sourceText, autodetectFormat, mnemonics)
 
 	@classmethod
-	def parseData(cls, dataBytes,
+	def parseText(cls, sourceText,
 		      autodetectFormat=True,
-		      mnemonics=S7CPUSpecs.MNEMONICS_AUTO):
+		      mnemonics=S7CPUConfig.MNEMONICS_AUTO):
 		try:
-			if not dataBytes.strip():
+			if not sourceText.strip():
 				return SymbolTable()
 			if autodetectFormat:
 				for implCls in cls.implementations:
-					if implCls._probe(dataBytes):
+					if implCls._probe(sourceText):
 						parserClass = implCls
 						break
 				else:
@@ -316,28 +408,28 @@ class SymTabParser(object):
 						"symbol table parser")
 			else:
 				parserClass = cls
-			if mnemonics == S7CPUSpecs.MNEMONICS_AUTO:
-				instance = parserClass(S7CPUSpecs.MNEMONICS_EN)
+			if mnemonics == S7CPUConfig.MNEMONICS_AUTO:
+				instance = parserClass(S7CPUConfig.MNEMONICS_EN)
 				try:
-					symTab = instance._parse(dataBytes)
+					symTab = instance._parse(sourceText)
 				except AwlSimError as e:
-					instance = parserClass(S7CPUSpecs.MNEMONICS_DE)
-					symTab = instance._parse(dataBytes)
+					instance = parserClass(S7CPUConfig.MNEMONICS_DE)
+					symTab = instance._parse(sourceText)
 			else:
 				instance = parserClass(mnemonics)
-				symTab = instance._parse(dataBytes)
+				symTab = instance._parse(sourceText)
 			return symTab
 		except UnicodeError as e:
 			raise AwlSimError("Encoding error while trying to decode "
 				"symbol table.")
 
 	@classmethod
-	def _probe(cls, dataBytes):
+	def _probe(cls, sourceText):
 		try:
-			if not dataBytes.decode(cls.ENCODING).strip():
+			if not sourceText.strip():
 				return False
 			p = cls(None)
-			p._parse(dataBytes, probeOnly=True)
+			p._parse(sourceText, probeOnly=True)
 		except AwlSimError as e:
 			return False
 		except UnicodeError as e:
@@ -382,26 +474,53 @@ class SymTabParser(object):
 		return sym
 
 class SymTabParser_ASC(SymTabParser):
-	def _parse(self, dataBytes, probeOnly=False):
-		data = dataBytes.decode(self.ENCODING)
+	"""ASC symbol table file parser.
+	"""
+
+	# Character lengths of the fields.
+	# (The length field being restricted to 4 characters
+	#  is an Awlsim restriction.)
+	LEN_LEN		= 4
+	LEN_NAME	= 24
+	LEN_ADDR	= 12
+	LEN_TYPE	= 10
+	LEN_COMMENT	= 80
+
+	def _parse(self, data, probeOnly=False):
 		table = SymbolTable()
 		lines = data.splitlines()
 		for i, line in enumerate(lines):
 			lineNr = i + 1
 			if not line.strip():
 				continue
+			if not line.startswith("126,"):
+				# Technically it is allowed to have a different length
+				# prefix, but we do not support that.
+				# S7 and Awlsim do only generate entries with
+				# 126 characters, so we restrict the parser to that.
+				raise AwlSimError("ASC symbol table parser: "\
+					"Invalid line start (!= '126,') in "\
+					"line %d" % lineNr)
+			if len(line) >= self.LEN_LEN + self.LEN_NAME + self.LEN_ADDR + 1 and\
+			   len(line) < 130:
+				# As a convenience we support entries with the trailing
+				# whitespace after the data type being stripped.
+				# (We assume a data type length of at least 1 character here)
+				# Just add some white space to fill the 130 characters.
+				# This is an Awlsim extension.
+				line += " " * (130 - len(line))
 			if len(line) != 130:
 				raise AwlSimError("ASC symbol table parser: "\
 					"Invalid line length (!= 130 chars) in "\
 					"line %d" % lineNr)
-			if not line.startswith("126,"):
-				raise AwlSimError("ASC symbol table parser: "\
-					"Invalid line start (!= '126,') in "\
-					"line %d" % lineNr)
-			symName = line[4:28]
-			symAddr = line[28:40]
-			symType = line[40:50]
-			symComment = line[50:]
+			offs = self.LEN_LEN
+			symName = line[offs : offs + self.LEN_NAME]
+			offs += self.LEN_NAME
+			symAddr = line[offs : offs + self.LEN_ADDR]
+			offs += self.LEN_ADDR
+			symType = line[offs : offs + self.LEN_TYPE]
+			offs += self.LEN_TYPE
+			symComment = line[offs : ]
 			if not probeOnly:
 				table.add(self.parseSym(symName = symName,
 							symAddr = symAddr,
@@ -413,8 +532,10 @@ class SymTabParser_ASC(SymTabParser):
 SymTabParser.implementations.append(SymTabParser_ASC)
 
 class SymTabParser_CSV(SymTabParser):
-	def _parse(self, dataBytes, probeOnly=False):
-		data = dataBytes.decode(self.ENCODING)
+	"""CSV symbol table file parser.
+	"""
+
+	def _parse(self, data, probeOnly=False):
 		table = SymbolTable()
 		csvReader = csv.reader(data.splitlines(),
 				       dialect="awlsim_symtab")
