@@ -2,7 +2,7 @@
 #
 # AWL simulator - SFBs
 #
-# Copyright 2014 Michael Buesch <m@bues.ch>
+# Copyright 2014-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,40 +20,44 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.systemblocks.systemblocks import *
+from awlsim.common.datatypehelpers import * #+cimport
+from awlsim.common.exceptions import *
+
+from awlsim.core.systemblocks.systemblocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.datatypes import *
 from awlsim.core.util import *
 
 
-class SFB1(SFB):
+class SFB1(SFB): #+cdef
 	name = (1, "CTD", "IEC 1131-3 down counter")
 
 	interfaceFields = {
 		BlockInterfaceField.FTYPE_IN	: (
-			BlockInterfaceField(name = "CD",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "LOAD",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "PV",
-					    dataType = AwlDataType.makeByName("INT")),
+			BlockInterfaceField(name="CD", dataType="BOOL"),
+			BlockInterfaceField(name="LOAD", dataType="BOOL"),
+			BlockInterfaceField(name="PV", dataType="INT"),
 		),
 		BlockInterfaceField.FTYPE_OUT	: (
-			BlockInterfaceField(name = "Q",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "CV",
-					    dataType = AwlDataType.makeByName("INT")),
+			BlockInterfaceField(name="Q", dataType="BOOL"),
+			BlockInterfaceField(name="CV", dataType="INT"),
 		),
 		BlockInterfaceField.FTYPE_STAT	: (
-			BlockInterfaceField(name = "CDO",
-					    dataType = AwlDataType.makeByName("BOOL")),
+			BlockInterfaceField(name="CDO", dataType="BOOL"),
 		),
 	}
 
-	def run(self):
+	def run(self): #+cpdef
+#@cy		cdef S7StatusWord s
+
+		s = self.cpu.statusWord
+
 		# CD pos-edge detection
 		CD = self.fetchInterfaceFieldByName("CD")
-		CD_pos_edge = CD & ~self.fetchInterfaceFieldByName("CDO") & 1
+		CD_pos_edge = CD & (self.fetchInterfaceFieldByName("CDO") ^ 1) & 1
 		self.storeInterfaceFieldByName("CDO", CD)
 
 		CV = wordToSignedPyInt(self.fetchInterfaceFieldByName("CV"))
@@ -66,3 +70,5 @@ class SFB1(SFB):
 
 		# Update Q-status
 		self.storeInterfaceFieldByName("Q", 1 if CV <= 0 else 0)
+
+		s.BIE = 1
