@@ -2,7 +2,7 @@
 #
 # AWL simulator - SFBs
 #
-# Copyright 2014 Michael Buesch <m@bues.ch>
+# Copyright 2014-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,53 +20,53 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.systemblocks.systemblocks import *
+from awlsim.common.datatypehelpers import * #+cimport
+from awlsim.common.exceptions import *
+
+from awlsim.core.systemblocks.systemblocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.datatypes import *
 from awlsim.core.util import *
 
 
-class SFB2(SFB):
+class SFB2(SFB): #+cdef
 	name = (2, "CTUD", "IEC 1131-3 up/down counter")
 
 	interfaceFields = {
 		BlockInterfaceField.FTYPE_IN	: (
-			BlockInterfaceField(name = "CU",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "CD",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "R",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "LOAD",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "PV",
-					    dataType = AwlDataType.makeByName("INT")),
+			BlockInterfaceField(name="CU", dataType="BOOL"),
+			BlockInterfaceField(name="CD", dataType="BOOL"),
+			BlockInterfaceField(name="R", dataType="BOOL"),
+			BlockInterfaceField(name="LOAD", dataType="BOOL"),
+			BlockInterfaceField(name="PV", dataType="INT"),
 		),
 		BlockInterfaceField.FTYPE_OUT	: (
-			BlockInterfaceField(name = "QU",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "QD",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "CV",
-					    dataType = AwlDataType.makeByName("INT")),
+			BlockInterfaceField(name="QU", dataType="BOOL"),
+			BlockInterfaceField(name="QD", dataType="BOOL"),
+			BlockInterfaceField(name="CV", dataType="INT"),
 		),
 		BlockInterfaceField.FTYPE_STAT	: (
-			BlockInterfaceField(name = "CUO",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "CDO",
-					    dataType = AwlDataType.makeByName("BOOL")),
+			BlockInterfaceField(name="CUO", dataType="BOOL"),
+			BlockInterfaceField(name="CDO", dataType="BOOL"),
 		),
 	}
 
-	def run(self):
+	def run(self): #+cpdef
+#@cy		cdef S7StatusWord s
+
+		s = self.cpu.statusWord
+
 		# CU pos-edge detection
 		CU = self.fetchInterfaceFieldByName("CU")
-		CU_pos_edge = CU & ~self.fetchInterfaceFieldByName("CUO") & 1
+		CU_pos_edge = CU & (self.fetchInterfaceFieldByName("CUO") ^ 1) & 1
 		self.storeInterfaceFieldByName("CUO", CU)
 
 		# CD pos-edge detection
 		CD = self.fetchInterfaceFieldByName("CD")
-		CD_pos_edge = CD & ~self.fetchInterfaceFieldByName("CDO") & 1
+		CD_pos_edge = CD & (self.fetchInterfaceFieldByName("CDO") ^ 1) & 1
 		self.storeInterfaceFieldByName("CDO", CD)
 
 		# Count
@@ -88,3 +88,5 @@ class SFB2(SFB):
 		# Update Q-status
 		self.storeInterfaceFieldByName("QU", 1 if CV >= PV else 0)
 		self.storeInterfaceFieldByName("QD", 1 if CV <= 0 else 0)
+
+		s.BIE = 1
