@@ -2,7 +2,7 @@
 #
 # AWL simulator - SFBs
 #
-# Copyright 2014 Michael Buesch <m@bues.ch>
+# Copyright 2014-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,35 +20,34 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.systemblocks.systemblocks import *
+from awlsim.common.datatypehelpers import * #+cimport
+from awlsim.common.exceptions import *
+
+from awlsim.core.systemblocks.systemblocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.datatypes import *
 from awlsim.core.util import *
 
 
-class SFB3(SFB):
+class SFB3(SFB): #+cdef
 	name = (3, "TP", "IEC 1131-3 timed pulse")
 
 	interfaceFields = {
 		BlockInterfaceField.FTYPE_IN	: (
-			BlockInterfaceField(name = "IN",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "PT",
-					    dataType = AwlDataType.makeByName("TIME")),
+			BlockInterfaceField(name="IN", dataType="BOOL"),
+			BlockInterfaceField(name="PT", dataType="TIME"),
 		),
 		BlockInterfaceField.FTYPE_OUT	: (
-			BlockInterfaceField(name = "Q",
-					    dataType = AwlDataType.makeByName("BOOL")),
-			BlockInterfaceField(name = "ET",
-					    dataType = AwlDataType.makeByName("TIME")),
+			BlockInterfaceField(name="Q", dataType="BOOL"),
+			BlockInterfaceField(name="ET", dataType="TIME"),
 		),
 		BlockInterfaceField.FTYPE_STAT	: (
-			BlockInterfaceField(name = "STATE",
-					    dataType = AwlDataType.makeByName("BYTE")),
-			BlockInterfaceField(name = "STIME",
-					    dataType = AwlDataType.makeByName("TIME")),
-			BlockInterfaceField(name = "ATIME",
-					    dataType = AwlDataType.makeByName("TIME")),
+			BlockInterfaceField(name="STATE", dataType="BYTE"),
+			BlockInterfaceField(name="STIME", dataType="TIME"),
+			BlockInterfaceField(name="ATIME", dataType="TIME"),
 		),
 	}
 
@@ -56,7 +55,12 @@ class SFB3(SFB):
 	STATE_RUNNING		= 1 << 0
 	STATE_FINISHED		= 1 << 1
 
-	def run(self):
+	def run(self): #+cpdef
+#@cy		cdef S7StatusWord s
+
+		s = self.cpu.statusWord
+		s.BIE = 1
+
 		PT = dwordToSignedPyInt(self.fetchInterfaceFieldByName("PT"))
 		if PT <= 0:
 			# Invalid PT. Abort and reset state.
@@ -64,13 +68,16 @@ class SFB3(SFB):
 			if PT == 0:
 				# S7 resets IN here, for whatever weird reason.
 				self.storeInterfaceFieldByName("IN", 0)
+			else:
+				# Negative PT. This is an error.
+				s.BIE = 0
 			self.storeInterfaceFieldByName("Q", 0)
 			self.storeInterfaceFieldByName("ET", 0)
 			self.storeInterfaceFieldByName("STATE", 0)
 			return
 
 		# Get the current time, as S7-time value (31-bit milliseconds)
-		ATIME = int(self.cpu.now * 1000.0) & 0x7FFFFFFF
+		ATIME = self.cpu.now_TIME
 
 		STATE = self.fetchInterfaceFieldByName("STATE")
 		IN = self.fetchInterfaceFieldByName("IN")
