@@ -2,7 +2,7 @@
 #
 # AWL simulator - SFBs
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +20,22 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.systemblocks.systemblocks import *
+from awlsim.common.exceptions import *
+
+from awlsim.core.systemblocks.systemblocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.datatypes import *
 from awlsim.core.util import *
 
 
-class SFBm1(SFB):
+class SFBm1(SFB): #+cdef
 	name = (-1, "__SFB_NOP", None)
 
-	def run(self):
-		pass # No operation
+	def run(self): #+cpdef
+#@cy		cdef S7StatusWord s
+
+		s = self.cpu.statusWord
+		s.BIE = 1
