@@ -2,7 +2,7 @@
 #
 # AWL simulator - SFCs
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,24 +20,32 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.systemblocks.systemblocks import *
+from awlsim.common.exceptions import *
+
+from awlsim.core.systemblocks.systemblocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.datatypes import *
 from awlsim.core.util import *
 
 
-class SFC64(SFC):
+class SFC64(SFC): #+cdef
 	name = (64, "TIME_TCK", "time tick")
 
 	interfaceFields = {
 		BlockInterfaceField.FTYPE_OUT	: (
-			BlockInterfaceField(name = "RET_VAL",
-					    dataType = AwlDataType.makeByName("TIME")),
+			BlockInterfaceField(name="RET_VAL", dataType="TIME"),
 		)
 	}
 
-	def run(self):
+	def run(self): #+cpdef
+#@cy		cdef S7StatusWord s
+
+		s = self.cpu.statusWord
+
 		# Return a 31-bit millisecond representation of "now".
 		self.cpu.updateTimestamp()
-		self.storeInterfaceFieldByName("RET_VAL",
-			int(self.cpu.now * 1000) & 0x7FFFFFFF)
+		self.storeInterfaceFieldByName("RET_VAL", self.cpu.now_TIME)
+		s.BIE = 1
