@@ -2,7 +2,7 @@
 #
 # AWL simulator - SFCs
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,23 +20,28 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-from awlsim.core.systemblocks.systemblocks import *
+from awlsim.common.datatypehelpers import * #+cimport
+from awlsim.common.exceptions import *
+
+from awlsim.core.systemblocks.systemblocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.datatypes import *
 from awlsim.core.util import *
 
 
-class SFCm2(SFC):
+class SFCm2(SFC): #+cdef
 	name = (-2, "__REBOOT", None)
 
 	interfaceFields = {
 		BlockInterfaceField.FTYPE_IN	: (
-			BlockInterfaceField(name = "REBOOT_TYPE",
-					    dataType = AwlDataType.makeByName("INT")),
+			BlockInterfaceField(name="REBOOT_TYPE", dataType="INT"),
 		),
 	}
 
-	def run(self):
+	def run(self): #+cpdef
 		rebootType = wordToSignedPyInt(self.fetchInterfaceFieldByName("REBOOT_TYPE"))
 		if rebootType == 1:
 			raise MaintenanceRequest(MaintenanceRequest.TYPE_SOFTREBOOT,
