@@ -2,7 +2,7 @@
 #
 # AWL simulator - System-blocks
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,21 +20,31 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
-#from awlsim.core.instructions.insn_generic_call cimport * #@cy
+from awlsim.common.exceptions import *
 
-from awlsim.core.instructions.insn_generic_call import * #@nocy
-from awlsim.core.blocks import *
-from awlsim.core.translator import *
+from awlsim.core.instructions.insn_generic_call import * #+cimport
+from awlsim.core.systemblocks.error_codes import *
+from awlsim.core.systemblocks.systemblockinterface import *
+from awlsim.core.blocks import * #+cimport
+from awlsim.core.blockinterface import *
+from awlsim.core.identifier import *
+from awlsim.core.offset import * #+cimport
+from awlsim.core.operatortypes import * #+cimport
+from awlsim.core.operators import * #+cimport
+
+from awlsim.awlcompiler.translator import *
 
 
-class SystemBlock(StaticCodeBlock):
+class SystemBlock(StaticCodeBlock): #+cdef
 	# The block identification. To be overridden by the subclass.
 	# The tuple is: (number, name, short_description)
 	name = (-1, "<unknown>", None)
 
-	isSystemBlock = True
+	BLOCKTYPESTR	= "SystemBlock"
+	_isSystemBlock	= True
 
 	def __init__(self, cpu, interface):
 		insns = [
@@ -42,18 +52,23 @@ class SystemBlock(StaticCodeBlock):
 		]
 		StaticCodeBlock.__init__(self, insns, self.name[0], interface)
 		self.cpu = cpu
+		self.__widthMaskAll = AwlOperatorWidths.WIDTH_MASK_ALL
 
-	def run(self):
+	def run(self): #+cpdef
 		# Reimplement this method
 		raise NotImplementedError
 
 	# Fetch the value of a block-interface field.
-	def fetchInterfaceFieldByName(self, name):
-		return self.cpu.fetch(self.__interfaceOpers[name])
+	def fetchInterfaceFieldByName(self, name): #@nocy
+#@cy	cdef object fetchInterfaceFieldByName(self, object name):
+		return self.cpu.fetch(self.__interfaceOpers[name],
+				      self.__widthMaskAll)
 
 	# Store a value to a block-interface field.
-	def storeInterfaceFieldByName(self, name, value):
-		return self.cpu.store(self.__interfaceOpers[name], value)
+	def storeInterfaceFieldByName(self, name, value): #@nocy
+#@cy	cdef storeInterfaceFieldByName(self, object name, object value):
+		return self.cpu.store(self.__interfaceOpers[name], value,
+				      self.__widthMaskAll)
 
 	# Resolve hard wired symbolic accesses
 	# (i.e. accesses not done in AWL instructions)
@@ -63,36 +78,32 @@ class SystemBlock(StaticCodeBlock):
 		self.__interfaceOpers = {}
 		for field in self.interface.fields_IN_OUT_INOUT_STAT:
 			# Create a scratch-operator for the access.
-			offset = AwlOffset(None, None)
-			offset.varName = field.name
-			oper = AwlOperator(AwlOperator.NAMED_LOCAL, 0,
-					   offset)
+			offset = make_AwlOffset(0, 0)
+			offset.identChain = AwlDataIdentChain.parseString(field.name)
+			oper = make_AwlOperator(AwlOperatorTypes.NAMED_LOCAL, 0,
+					   offset, None)
 			# Resolve the scratch-operator.
 			oper = resolver.resolveNamedLocal(block=self, insn=None,
 							  oper=oper, pointer=False)
 			# Store the scratch operator for later use.
 			self.__interfaceOpers[field.name] = oper
 
-class SFBInterface(FBInterface):
-	pass
+class SFB(SystemBlock): #+cdef
+	"""SFB base class.
+	"""
 
-class SFB(SystemBlock):
-	isFB = True
+	BLOCKTYPESTR	= "SFB"
+	_isFB = True
 
 	def __init__(self, cpu):
 		SystemBlock.__init__(self, cpu, SFBInterface())
 
-	def __repr__(self):
-		return "SFB %d" % self.index
+class SFC(SystemBlock): #+cdef
+	"""SFC base class.
+	"""
 
-class SFCInterface(FCInterface):
-	pass
-
-class SFC(SystemBlock):
-	isFC = True
+	BLOCKTYPESTR	= "SFC"
+	_isFC = True
 
 	def __init__(self, cpu):
 		SystemBlock.__init__(self, cpu, SFCInterface())
-
-	def __repr__(self):
-		return "SFC %d" % self.index
