@@ -2,7 +2,7 @@
 #
 # AWL simulator - PLC core server client
 #
-# Copyright 2013-2014 Michael Buesch <m@bues.ch>
+# Copyright 2013-2017 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +20,19 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 from awlsim.common.util import *
-from awlsim.common.subprocess import *
+from awlsim.common.subprocess_wrapper import *
+from awlsim.common.net import *
+from awlsim.common.exceptions import *
+from awlsim.common.monotonic import * #+cimport
+
+from awlsim.coreclient.sshtunnel import *
+from awlsim.coreclient.util import *
 
 from awlsim.coreserver.server import *
-from awlsim.coreserver.messages import *
-from awlsim.coreserver.memarea import *
 
 import sys
 import socket
@@ -35,18 +40,44 @@ import errno
 import time
 
 
+class MsgWaiter(object):
+	"""Message waiter queue entry.
+	"""
+
+	def __init__(self, checkCallback):
+		self.checkCallback = checkCallback
+		self.rxMsg = None
+
+	def receiveMsg(self, rxMsg):
+		if rxMsg is None:
+			return False
+		if self.rxMsg is None:
+			if self.checkCallback(rxMsg):
+				self.rxMsg = rxMsg
+				return True
+		return False
+
 class AwlSimClient(object):
+	"""Awlsim coreserver client API.
+	"""
+
 	def __init__(self):
 		self.serverProcess = None
 		self.serverProcessHost = None
 		self.serverProcessPort = None
 		self.__transceiver = None
+		self.__defaultTimeout = 3.0
+		self.__msgWaiters = []
 
 	def spawnServer(self,
 			interpreter=None,
 			serverExecutable=None,
 			listenHost=AwlSimServer.DEFAULT_HOST,
-			listenPort=AwlSimServer.DEFAULT_PORT):
+			listenPort=AwlSimServer.DEFAULT_PORT,
+			frozenExecutableMagic=True,
+			commandMask=AwlSimServer.CMDMSK_DEFAULT,
+			projectFile=None,
+			projectWriteBack=False):
 		"""Spawn a new AwlSim-core server process.
 		interpreter -> The python interpreter to use. Must be either:
 			       - None: Use sys.executable as interpreter.
@@ -58,40 +89,88 @@ class AwlSimClient(object):
 				    May be a list of strings.
 		listenHost -> The hostname or IP address to listen on.
 		listenPort -> The port to listen on.
+			      This may be an iterable to try multiple ports.
+		frozenExecutableMagic -> If True and if running frozen executable,
+					 override serverExecutable.
+		commandMask -> Command mask for the server.
+		projectFile -> Project file for the server (if any).
+		projectWriteBack -> Enable write-back support for project (default off).
+
 		Returns the spawned process' PID."""
 
 		if self.serverProcess:
 			raise AwlSimError("Server already running")
 
-		if serverExecutable:
-			for serverExe in toList(serverExecutable):
-				if not findExecutable(serverExe):
-					continue
-				self.serverProcess = AwlSimServer.start(listenHost = listenHost,
-									listenPort = listenPort,
-									forkServerProcess = serverExe)
-				break
+		if frozenExecutableMagic and isWinStandalone:
+			# We are running the frozen executable
+			# and the magic is turned on.
+			# Enforce the server executable name.
+			oldExecutable = serverExecutable
+			serverExecutable = standaloneServerExe
+			if oldExecutable:
+				printWarning("spawnServer: Overriding server "
+					     "executable '%s' -> '%s'" %(
+					     oldExecutable, serverExecutable))
+
+		actualListenPort = None
+		for port in toList(listenPort):
+			if not netPortIsUnused(listenHost, port):
+				continue
+			if serverExecutable:
+				for serverExe in toList(serverExecutable):
+					if not findExecutable(serverExe):
+						continue
+					try:
+						self.serverProcess = AwlSimServer.start(
+							listenHost=listenHost,
+							listenPort=port,
+							forkServerProcess=serverExe,
+							commandMask=commandMask,
+							projectFile=projectFile,
+							projectWriteBack=projectWriteBack)
+					except AwlSimError as e:
+						if not isiterable(listenPort):
+							raise e
+					else:
+						actualListenPort = port
+					break
+				else:
+					raise AwlSimError("Unable to fork any of the supplied "
+						"server executables: %s" %\
+						str(toList(serverExecutable)))
 			else:
-				raise AwlSimError("Unable to fork any of the supplied "
-					"server executables: %s" %\
-					str(toList(serverExecutable)))
+				if interpreter is None:
+					interpreter = sys.executable
+				for interp in toList(interpreter):
+					if not findExecutable(interp):
+						continue
+					try:
+						self.serverProcess=AwlSimServer.start(
+							listenHost=listenHost,
+							listenPort=port,
+							forkInterpreter=interp,
+							commandMask=commandMask,
+							projectFile=projectFile,
+							projectWriteBack=projectWriteBack)
+					except AwlSimError as e:
+						if not isiterable(listenPort):
+							raise e
+					else:
+						actualListenPort = port
+					break
+				else:
+					raise AwlSimError("Unable to fork an awlsim core server with "
+						"any of the supplied Python interpreters: %s\n"
+						"No interpreter found." %\
+						str(toList(interpreter)))
+			if actualListenPort:
+				break
 		else:
-			if interpreter is None:
-				interpreter = sys.executable
-			for interp in toList(interpreter):
-				if not findExecutable(interp):
-					continue
-				self.serverProcess = AwlSimServer.start(listenHost = listenHost,
-									listenPort = listenPort,
-									forkInterpreter = interp)
-				break
-			else:
-				raise AwlSimError("Unable to fork an awlsim core server with "
-					"any of the supplied Python interpreters: %s\n"
-					"No interpreter found." %\
-					str(toList(interpreter)))
+			raise AwlSimError("Unable to find a port to spawn the "
+				"awlsim core server on.\nTried port %d to %d on '%s'." %(
+				toList(listenPort)[0], toList(listenPort)[-1], listenHost))
 		self.serverProcessHost = listenHost
-		self.serverProcessPort = listenPort
+		self.serverProcessPort = actualListenPort
 		if isJython:
 			#XXX Workaround: Jython's socket module does not like connecting
 			# to a starting server. Wait a few seconds for the server
@@ -105,14 +184,12 @@ class AwlSimClient(object):
 			return
 
 		if self.__transceiver:
-			try:
+			with contextlib.suppress(AwlSimError, MaintenanceRequest):
 				msg = AwlSimMessage_SHUTDOWN()
 				status = self.__sendAndWaitFor_REPLY(msg)
 				if status != AwlSimMessage_REPLY.STAT_OK:
 					printError("AwlSimClient: Failed to shut "
 						"down server via message")
-			except (AwlSimError, MaintenanceRequest) as e:
-				pass
 
 		self.serverProcess.terminate()
 		self.serverProcess.wait()
@@ -128,14 +205,17 @@ class AwlSimClient(object):
 		host -> The hostname or IP address to connect to.
 		port -> The port to connect to."""
 
+		self.__defaultTimeout = timeout
 		startTime = monotonic_time()
 		readableSockaddr = host
+		sock, ok = None, False
+		_SocketErrors = SocketErrors
 		try:
-			family, socktype, sockaddr = AwlSimServer.getaddrinfo(host, port)
+			family, socktype, sockaddr = netGetAddrInfo(host, port)
 			if family == AF_UNIX:
 				readableSockaddr = sockaddr
 			else:
-				readableSockaddr = "%s:%d" % (sockaddr[0], sockaddr[1])
+				readableSockaddr = "[%s]:%d" % (sockaddr[0], sockaddr[1])
 			printInfo("AwlSimClient: Connecting to server '%s'..." % readableSockaddr)
 			sock = socket.socket(family, socktype)
 			while 1:
@@ -144,7 +224,7 @@ class AwlSimClient(object):
 						"to AwlSimServer %s" % readableSockaddr)
 				try:
 					sock.connect(sockaddr)
-				except SocketErrors as e:
+				except _SocketErrors as e:
 					if e.errno == errno.ECONNREFUSED or\
 					   e.errno == errno.ENOENT:
 						self.sleep(0.1)
@@ -156,17 +236,24 @@ class AwlSimClient(object):
 						continue
 					raise
 				break
-		except SocketErrors as e:
+			ok = True
+		except _SocketErrors as e:
 			raise AwlSimError("Failed to connect to AwlSimServer %s: %s" %\
 				(readableSockaddr, str(e)))
+		finally:
+			if not ok and sock:
+				with suppressAllExc:
+					sock.shutdown(socket.SHUT_RDWR)
+				with suppressAllExc:
+					sock.close()
 		printInfo("AwlSimClient: Connected.")
 		self.__transceiver = AwlSimMessageTransceiver(sock, readableSockaddr)
-		self.lastRxMsg = None
+		self.__msgWaiters = []
 
 		# Ping the server
 		try:
 			self.__transceiver.send(AwlSimMessage_PING())
-			msg = self.__transceiver.receive(timeout = 2.0)
+			msg = self.__transceiver.receive(timeout = timeout)
 			if not msg:
 				raise AwlSimError("AwlSimClient: Server did not "
 					"respond to PING request.")
@@ -196,8 +283,11 @@ class AwlSimClient(object):
 	def __rx_NOP(self, msg):
 		pass # Nothing
 
+	def handle_EXCEPTION(self, exception):
+		raise exception
+
 	def __rx_EXCEPTION(self, msg):
-		raise msg.exception
+		self.handle_EXCEPTION(msg.exception)
 
 	def __rx_PING(self, msg):
 		self.__send(AwlSimMessage_PONG())
@@ -208,6 +298,18 @@ class AwlSimClient(object):
 	def __rx_PONG(self, msg):
 		self.handle_PONG()
 
+	def handle_AWLSRC(self, awlSource):
+		pass # Don't do anything by default
+
+	def __rx_AWLSRC(self, msg):
+		self.handle_AWLSRC(msg.source)
+
+	def handle_SYMTABSRC(self, symTabSource):
+		pass # Don't do anything by default
+
+	def __rx_SYMTABSRC(self, msg):
+		self.handle_SYMTABSRC(msg.source)
+
 	def handle_CPUDUMP(self, dumpText):
 		pass # Don't do anything by default
 
@@ -239,18 +341,28 @@ class AwlSimClient(object):
 	def __rx_IDENTS(self, msg):
 		self.handle_IDENTS(msg)
 
+	def handle_BLOCKINFO(self, msg):
+		pass # Don't do anything by default
+
+	def __rx_BLOCKINFO(self, msg):
+		self.handle_BLOCKINFO(msg)
+
 	__msgRxHandlers = {
 		AwlSimMessage.MSG_ID_REPLY		: __rx_NOP,
 		AwlSimMessage.MSG_ID_EXCEPTION		: __rx_EXCEPTION,
+		AwlSimMessage.MSG_ID_MAINTREQ		: __rx_MAINTREQ,
 		AwlSimMessage.MSG_ID_PING		: __rx_PING,
 		AwlSimMessage.MSG_ID_PONG		: __rx_PONG,
-		AwlSimMessage.MSG_ID_CPUDUMP		: __rx_CPUDUMP,
-		AwlSimMessage.MSG_ID_MAINTREQ		: __rx_MAINTREQ,
+		AwlSimMessage.MSG_ID_AWLSRC		: __rx_AWLSRC,
+		AwlSimMessage.MSG_ID_SYMTABSRC		: __rx_SYMTABSRC,
+		AwlSimMessage.MSG_ID_IDENTS		: __rx_IDENTS,
+		AwlSimMessage.MSG_ID_BLOCKINFO		: __rx_BLOCKINFO,
 		AwlSimMessage.MSG_ID_CPUSPECS		: __rx_NOP,
+		AwlSimMessage.MSG_ID_CPUCONF		: __rx_NOP,
+		AwlSimMessage.MSG_ID_RUNSTATE		: __rx_NOP,
+		AwlSimMessage.MSG_ID_CPUDUMP		: __rx_CPUDUMP,
 		AwlSimMessage.MSG_ID_MEMORY		: __rx_MEMORY,
 		AwlSimMessage.MSG_ID_INSNSTATE		: __rx_INSNSTATE,
-		AwlSimMessage.MSG_ID_RUNSTATE		: __rx_NOP,
-		AwlSimMessage.MSG_ID_IDENTS		: __rx_IDENTS,
 	}
 
 	# Main message processing
@@ -258,7 +370,6 @@ class AwlSimClient(object):
 	#          0 -> No timeout (= Nonblocking). Return immediately.
 	#          x -> Timeout, in seconds.
 	def processMessages(self, timeout=None):
-		self.lastRxMsg = None
 		if not self.__transceiver:
 			return False
 		try:
@@ -277,19 +388,24 @@ class AwlSimClient(object):
 				self.__transceiver.peerInfoString)
 		if not msg:
 			return False
-		self.lastRxMsg = msg
-		try:
-			handler = self.__msgRxHandlers[msg.msgId]
-		except KeyError:
-			raise AwlSimError("AwlSimClient: Received unsupported "
-				"message 0x%02X" % msg.msgId)
-		handler(self, msg)
+		printDebug("AwlSimClient: Received message 0x%04X" % msg.msgId)
+		for waiter in self.__msgWaiters:
+			if waiter.receiveMsg(msg):
+				break
+		else:
+			try:
+				handler = self.__msgRxHandlers[msg.msgId]
+			except KeyError:
+				raise AwlSimError("AwlSimClient: Received unsupported "
+					"message 0x%02X" % msg.msgId)
+			handler(self, msg)
 		return True
 
 	def sleep(self, seconds):
 		time.sleep(seconds)
 
 	def __send(self, txMsg):
+		printDebug("AwlSimClient: Sending message 0x%04X" % txMsg.msgId)
 		try:
 			self.__transceiver.send(txMsg)
 		except TransferError as e:
@@ -300,24 +416,29 @@ class AwlSimClient(object):
 				 str(e), str(e.errno)))
 
 	def __sendAndWait(self, txMsg, checkRxMsg,
-			  waitTimeout=3.0,
+			  waitTimeout=None,
 			  ignoreMaintenanceRequests=False):
-		self.__send(txMsg)
-		now = monotonic_time()
-		end = now + waitTimeout
-		while now < end:
-			try:
-				if self.processMessages(0.1):
-					rxMsg = self.lastRxMsg
-					if checkRxMsg(rxMsg):
-						return rxMsg
-			except MaintenanceRequest as e:
-				if not ignoreMaintenanceRequests:
-					raise e
+		waiter = MsgWaiter(checkRxMsg)
+		self.__msgWaiters.append(waiter)
+		try:
+			self.__send(txMsg)
 			now = monotonic_time()
-		raise AwlSimError("AwlSimClient: Timeout waiting for server reply.")
+			end = now + (self.__defaultTimeout if waitTimeout is None\
+				     else waitTimeout)
+			while now < end:
+				try:
+					if self.processMessages(0.1):
+						if waiter.rxMsg is not None:
+							return waiter.rxMsg
+				except MaintenanceRequest as e:
+					if not ignoreMaintenanceRequests:
+						raise e
+				now = monotonic_time()
+			raise AwlSimError("AwlSimClient: Timeout waiting for server reply.")
+		finally:
+			self.__msgWaiters.remove(waiter)
 
-	def __sendAndWaitFor_REPLY(self, msg, timeout=3.0,
+	def __sendAndWaitFor_REPLY(self, msg, timeout=None,
 				   ignoreMaintenanceRequests=False):
 		def checkRxMsg(rxMsg):
 			return rxMsg.msgId == AwlSimMessage.MSG_ID_REPLY and\
@@ -360,63 +481,192 @@ class AwlSimClient(object):
 			return True
 		return False
 
-	def loadCode(self, codeSource):
+	def getAwlSource(self, identHash, sync=True):
 		if not self.__transceiver:
 			return False
-		msg = AwlSimMessage_LOAD_CODE(codeSource)
-		status = self.__sendAndWaitFor_REPLY(msg, 10.0)
-		if status != AwlSimMessage_REPLY.STAT_OK:
-			raise AwlSimError("AwlSimClient: Failed to load code")
+		msg = AwlSimMessage_GET_AWLSRC(identHash)
+		if sync:
+			rxMsg = self.__sendAndWait(msg,
+				lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_AWLSRC)
+			return rxMsg.source
+		else:
+			self.__send(msg)
 		return True
 
-	def loadSymbolTable(self, symTabSource):
+	def loadAwlSource(self, awlSource):
 		if not self.__transceiver:
 			return False
-		msg = AwlSimMessage_LOAD_SYMTAB(symTabSource)
-		status = self.__sendAndWaitFor_REPLY(msg)
+		msg = AwlSimMessage_AWLSRC(awlSource)
+		self.__transceiver.txCork(True)
+		try:
+			status = self.__sendAndWaitFor_REPLY(msg, 10.0)
+		finally:
+			self.__transceiver.txCork(False)
 		if status != AwlSimMessage_REPLY.STAT_OK:
-			raise AwlSimError("AwlSimClient: Failed to load symbol table")
+			raise AwlSimError("AwlSimClient: Failed to load AWL source")
 		return True
 
+	def loadFupSource(self, fupSource):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_FUPSRC(fupSource)
+		self.__transceiver.txCork(True)
+		try:
+			status = self.__sendAndWaitFor_REPLY(msg, 10.0)
+		finally:
+			self.__transceiver.txCork(False)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to load FUP source")
+		return True
+
+	def loadKopSource(self, kopSource):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_KOPSRC(kopSource)
+		self.__transceiver.txCork(True)
+		try:
+			status = self.__sendAndWaitFor_REPLY(msg, 10.0)
+		finally:
+			self.__transceiver.txCork(False)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to load FUP source")
+		return True
+
+	def loadAwlSources(self, awlSources):
+		return all(self.loadAwlSource(awlSource)
+			   for awlSource in awlSources)
+
+	def loadFupSources(self, fupSources):
+		return all(self.loadFupSource(fupSource)
+			   for fupSource in fupSources)
+
+	def loadKopSources(self, kopSources):
+		return all(self.loadKopSource(kopSource)
+			   for kopSource in kopSources)
+
+	def getSymTabSource(self, identHash, sync=True):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_GET_SYMTABSRC(identHash)
+		if sync:
+			rxMsg = self.__sendAndWait(msg,
+				lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_SYMTABSRC)
+			return rxMsg.source
+		else:
+			self.__send(msg)
+		return True
+
+	def loadSymTabSource(self, symTabSource):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_SYMTABSRC(symTabSource)
+		self.__transceiver.txCork(True)
+		try:
+			status = self.__sendAndWaitFor_REPLY(msg)
+		finally:
+			self.__transceiver.txCork(False)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to load symbol table source")
+		return True
+
+	def loadSymTabSources(self, symTabSources):
+		return all(self.loadSymTabSource(symTabSource)
+			   for symTabSource in symTabSources)
+
 	def loadLibraryBlock(self, libSelection):
 		if not self.__transceiver:
 			return False
-		msg = AwlSimMessage_LOAD_LIB(libSelection)
+		msg = AwlSimMessage_LIBSEL(libSelection)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
-			raise AwlSimError("AwlSimClient: Failed to load library block")
+			raise AwlSimError("AwlSimClient: Failed to load library block selection")
 		return True
 
-	def loadHardwareModule(self, name, parameters={}):
+	def loadLibraryBlocks(self, libSelections):
+		return all(self.loadLibraryBlock(libSel)
+			   for libSel in libSelections)
+
+	def loadHardwareModule(self, hwmodDesc):
 		if not self.__transceiver:
 			return False
-		msg = AwlSimMessage_LOAD_HW(name = name,
-					    paramDict = parameters)
+		msg = AwlSimMessage_HWMOD(hwmodDesc)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load hardware module")
 		return True
 
+	def loadHardwareModules(self, hwmodDescs):
+		return all(self.loadHardwareModule(hwmodDesc)
+			   for hwmodDesc in hwmodDescs)
+
+	def build(self):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_BUILD()
+		status = self.__sendAndWaitFor_REPLY(msg)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to build sources")
+		return True
+
+	def removeSource(self, identHash):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_REMOVESRC(identHash)
+		status = self.__sendAndWaitFor_REPLY(msg)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to remove source")
+		return True
+
+	def removeBlock(self, blockInfo):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_REMOVEBLK(blockInfo)
+		status = self.__sendAndWaitFor_REPLY(msg)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to remove block")
+		return True
+
 	# Request the (source) ident hashes from the CPU.
 	# This method is asynchronous.
 	# The idents are returned via handle_IDENTS()
-	def requestIdents(self, reqAwlSources = False,
-			  reqSymTabSources = False,
-			  reqHwModules = False,
-			  reqLibSelections = False):
+	def requestIdents(self,
+			  reqAwlSources=False,
+			  reqFupSources=False,
+			  reqKopSources=False,
+			  reqSymTabSources=False,
+			  reqHwModules=False,
+			  reqLibSelections=False):
 		if not self.__transceiver:
 			return False
 		self.__send(AwlSimMessage_GET_IDENTS(
 			(AwlSimMessage_GET_IDENTS.GET_AWLSRCS if reqAwlSources else 0) |\
+			(AwlSimMessage_GET_IDENTS.GET_FUPSRCS if reqFupSources else 0) |\
+			(AwlSimMessage_GET_IDENTS.GET_KOPSRCS if reqKopSources else 0) |\
 			(AwlSimMessage_GET_IDENTS.GET_SYMTABSRCS if reqSymTabSources else 0) |\
 			(AwlSimMessage_GET_IDENTS.GET_HWMODS if reqHwModules else 0) |\
 			(AwlSimMessage_GET_IDENTS.GET_LIBSELS if reqLibSelections else 0)))
 		return True
 
+	# Request the compiled block info from the CPU.
+	# This method is asynchronous.
+	# The idents are returned via handle_BLOCKINFO()
+	def requestBlockInfo(self, reqOBInfo = False,
+			     reqFCInfo = False,
+			     reqFBInfo = False,
+			     reqDBInfo = False):
+		if not self.__transceiver:
+			return False
+		self.__send(AwlSimMessage_GET_BLOCKINFO(
+			(AwlSimMessage_GET_BLOCKINFO.GET_OB_INFO if reqOBInfo else 0) |\
+			(AwlSimMessage_GET_BLOCKINFO.GET_FC_INFO if reqFCInfo else 0) |\
+			(AwlSimMessage_GET_BLOCKINFO.GET_FB_INFO if reqFBInfo else 0) |\
+			(AwlSimMessage_GET_BLOCKINFO.GET_DB_INFO if reqDBInfo else 0)))
+		return True
+
 	def __setOption(self, name, value, sync=True):
 		if not self.__transceiver:
 			return False
-		msg = AwlSimMessage_SET_OPT(name, str(value))
+		msg = AwlSimMessage_OPT(name, str(value))
 		if sync:
 			status = self.__sendAndWaitFor_REPLY(msg)
 			if status != AwlSimMessage_REPLY.STAT_OK:
@@ -425,25 +675,19 @@ class AwlSimClient(object):
 			self.__send(msg)
 		return True
 
-	def setLoglevel(self, level=Logging.LOG_INFO):
-		Logging.setLoglevel(level)
-		return self.__setOption("loglevel", int(level))
-
-	def enableOBTempPresets(self, enable=True):
-		return self.__setOption("ob_temp_presets", int(bool(enable)))
-
-	def enableExtendedInsns(self, enable=True):
-		return self.__setOption("extended_insns", int(bool(enable)))
+	def setLoglevel(self, level=Logging.LOG_INFO,
+			setClientSide=True,
+			setServerSide=True):
+		if setClientSide:
+			Logging.setLoglevel(level)
+		if setServerSide:
+			if not self.__setOption("loglevel", int(level)):
+				return False
+		return True
 
 	def setPeriodicDumpInterval(self, interval=0):
 		return self.__setOption("periodic_dump_int", int(interval))
 
-	def setCycleTimeLimit(self, seconds=5.0):
-		return self.__setOption("cycle_time_limit", float(seconds))
-
-	def setRunTimeLimit(self, seconds=0.0):
-		return self.__setOption("runtime_limit", float(seconds))
-
 	# Set instruction state dumping.
 	# fromLine, toLine is the range of AWL line numbers for which
 	# dumping is enabled.
@@ -486,17 +730,60 @@ class AwlSimClient(object):
 			raise AwlSimError("AwlSimClient: Failed to set cpuspecs")
 		return True
 
+	def getCpuConf(self):
+		if not self.__transceiver:
+			return None
+		msg = AwlSimMessage_GET_CPUCONF()
+		rxMsg = self.__sendAndWait(msg,
+			lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_CPUCONF)
+		return rxMsg.cpuconf
+
+	def setCpuConf(self, cpuconf):
+		if not self.__transceiver:
+			return False
+		msg = AwlSimMessage_CPUCONF(cpuconf)
+		status = self.__sendAndWaitFor_REPLY(msg)
+		if status != AwlSimMessage_REPLY.STAT_OK:
+			raise AwlSimError("AwlSimClient: Failed to set cpuconf")
+		return True
+
+	def loadProject(self, project,
+			loadCpuSpecs=True, loadCpuConf=True,
+			loadHwMods=True,
+			loadSymTabs=True, loadLibSelections=True,
+			loadSources=True,
+			loadFup=True, loadKop=True):
+		"""Load selected settings and sources from project.
+		"""
+		if loadCpuSpecs:
+			self.setCpuSpecs(project.getCpuSpecs())
+		if loadCpuConf:
+			self.setCpuConf(project.getCpuConf())
+		if loadHwMods:
+			self.loadHardwareModules(project.getHwmodSettings().getLoadedModules())
+		if loadSymTabs:
+			self.loadSymTabSources(project.getSymTabSources())
+		if loadLibSelections:
+			self.loadLibraryBlocks(project.getLibSelections())
+		if loadSources:
+			self.loadAwlSources(project.getAwlSources())
+		if loadFup:
+			self.loadFupSources(project.getFupSources())
+		if loadKop:
+			self.loadKopSources(project.getKopSources())
+
 	# Set the memory areas we are interested in receiving
 	# dumps for, in the server.
 	# memAreas is a list of MemoryArea instances.
-	# The repetitionFactor tells whether to
-	#  - only run the request once (repetitionFactor=0)
-	#  - repeat on every n'th cycle (repetitionFactor=n)
+	# The repetitionPeriod tells whether to
+	#  - only run the request once (repetitionneriod < 0.0)
+	#  - repeat every n'th second (repetitionFactor = n)
 	# If sync is true, wait for a reply from the server.
-	def setMemoryReadRequests(self, memAreas, repetitionFactor=0, sync=False):
+	def setMemoryReadRequests(self, memAreas, repetitionPeriod = -1.0,
+				  sync = False):
 		if not self.__transceiver:
 			return False
-		msg = AwlSimMessage_REQ_MEMORY(0, repetitionFactor, memAreas)
+		msg = AwlSimMessage_REQ_MEMORY(0, repetitionPeriod, memAreas)
 		if sync:
 			msg.flags |= msg.FLG_SYNC
 			status = self.__sendAndWaitFor_REPLY(msg)
