@@ -2,7 +2,7 @@
 #
 # AWL simulator - GUI CPU state widgets
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2015 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,9 +20,12 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 from awlsim.gui.valuelineedit import *
+from awlsim.gui.blocktreewidget import *
+from awlsim.gui.icons import *
 from awlsim.gui.util import *
 
 
@@ -32,6 +35,9 @@ class StateWindow(QWidget):
 	# Config-change (address, type, etc...) signal
 	configChanged = Signal(QWidget)
 
+	COLOR_ACTIVE		= "#F0F000"
+	COLOR_LIGHTACTIVE	= "#FFFFC0"
+
 	def __init__(self, client, parent=None):
 		QWidget.__init__(self, parent)
 		self.setLayout(QGridLayout(self))
@@ -39,17 +45,30 @@ class StateWindow(QWidget):
 		pixmap.fill(QColor(0, 0, 192))
 		self.setWindowIcon(QIcon(pixmap))
 		self.__client = client
+		self.__forceMinimumSize = False
 
-	def update(self):
-		parent = self.parent()
+	def __updateSize(self):
+		try:
+			parent = self.parent()
+		except RuntimeError:
+			return
 		if isinstance(parent, StateMdiSubWindow):
 			widget = parent
 		else:
 			widget = self
-		size, hint = widget.size(), widget.minimumSizeHint()
-		if size.width() < hint.width() or\
-		   size.height() < hint.height():
-			widget.resize(hint)
+		if self.__forceMinimumSize:
+			widget.resize(widget.minimumSizeHint())
+		else:
+			size, hint = widget.size(), widget.minimumSizeHint()
+			if size.width() < hint.width() or\
+			   size.height() < hint.height():
+				widget.resize(hint)
+		self.__forceMinimumSize = False
+
+	def _updateSize(self, forceMinimum = False):
+		if forceMinimum:
+			self.__forceMinimumSize = True
+		QTimer.singleShot(0, self.__updateSize)
 
 	# Get a list of MemoryArea instances for the memory
 	# areas covered by this window.
@@ -86,21 +105,21 @@ class StateWindow(QWidget):
 class State_CPU(StateWindow):
 	def __init__(self, client, parent=None):
 		StateWindow.__init__(self, client, parent)
-		self.setWindowTitle("CPU Details")
+		self.setWindowTitle("CPU overview")
+		self.setWindowIcon(getIcon("cpu"))
 
 		self.label = QLabel(self)
-		font = self.label.font()
-		font.setFamily("Mono")
-		font.setFixedPitch(True)
-		font.setKerning(False)
-		self.label.setFont(font)
+		self.label.setFont(getDefaultFixedFont(10))
+		self.label.setTextInteractionFlags(Qt.TextSelectableByMouse |
+						   Qt.TextSelectableByKeyboard)
 		self.layout().addWidget(self.label, 0, 0)
 
-		self.label.setText("No CPU status available, yet.")
+		self.label.setText("No CPU status available.\n"
+				   "Please put the CPU into RUN mode.")
 
 	def setDumpText(self, text):
 		self.label.setText(text)
-		self.update()
+		self._updateSize()
 
 class AbstractDisplayWidget(QWidget):
 	EnumGen.start
@@ -129,6 +148,7 @@ class AbstractDisplayWidget(QWidget):
 	def __init__(self, stateWindow, addrSpace, addr, width, db, parent=None):
 		QWidget.__init__(self, parent)
 		self.setLayout(QGridLayout(self))
+		self.layout().setContentsMargins(QMargins(0, 10, 0, 0))
 
 		self.stateWindow = stateWindow
 		self.addrSpace = addrSpace
@@ -136,6 +156,10 @@ class AbstractDisplayWidget(QWidget):
 		self.width = width
 		self.db = db
 
+	def clear(self):
+		for i in range(self.width // 8):
+			self.setByte(i, 0)
+
 	def get(self):
 		pass
 
@@ -157,34 +181,79 @@ class AbstractDisplayWidget(QWidget):
 
 class BitDisplayWidget(AbstractDisplayWidget):
 	def __init__(self, stateWindow, addrSpace, addr, width, db,
-		     parent=None,
-		     displayPushButtons=True):
+		     parent=None):
 		AbstractDisplayWidget.__init__(self, stateWindow, addrSpace,
 					       addr, width, db, parent)
 
+		self.__cbActColor = QColor(StateWindow.COLOR_ACTIVE)
+		self.__pbActColor = QColor(StateWindow.COLOR_ACTIVE)
+		self.__frActColor = QColor(StateWindow.COLOR_LIGHTACTIVE)
+
+		self.__frames = {}
 		self.cbs = {}
 		self.pbs = {}
 		self.prevButtonStates = {}
 		y = 0
 		for i in range(self.width - 1, -1, -1):
-			cb = QCheckBox(str(i), self)
-			self.layout().addWidget(cb, y + 0, (self.width - i - 1) % 8)
-			self.cbs[i] = cb
-			cb.stateChanged.connect(self.changed)
-			if displayPushButtons:
-				pb = QPushButton("", self)
-				self.layout().addWidget(pb, y + 1, (self.width - i - 1) % 8)
-				self.pbs[i] = pb
-				self.prevButtonStates[i] = False
-				pb.pressed.connect(self.__buttonUpdate)
-				pb.released.connect(self.__buttonUpdate)
-			if i and i % 8 == 0:
-				y += 2
+			frame = QFrame(self)
+			frame.setFrameStyle(QFrame.Panel | QFrame.Raised)
+			frame.setLineWidth(2)
+			frame.setLayout(QHBoxLayout())
+			frame.setAutoFillBackground(True)
+			frame.layout().setContentsMargins(QMargins(3, 0, 0, 0))
+			self.__frames[i] = frame
 
-		self.update()
+			pb = QPushButton(self)
+			pb.setMaximumSize(QSize(13, 13))
+			frame.layout().addWidget(pb)
+			self.pbs[i] = pb
+			self.prevButtonStates[i] = False
+			pb.pressed.connect(self.__buttonUpdate)
+			pb.released.connect(self.__buttonUpdate)
+
+			cb = QCheckBox(str(i), self)
+			cb.setAutoFillBackground(False)
+			frame.layout().addWidget(cb)
+			self.cbs[i] = cb
+
+			self.__cbOrigColor = cb.palette().color(QPalette.Base)
+			self.__pbOrigColor = pb.palette().color(QPalette.Button)
+			self.__frOrigColor = frame.palette().color(QPalette.Window)
+
+			cb.stateChanged.connect(self.changed)
+			cb.stateChanged.connect(
+				lambda state, i=i: self.__updateColors(i, state))
+			self.__updateColors(i, Qt.Unchecked)
+
+			self.layout().addWidget(frame, y, (self.width - i - 1) % 8)
+			if i and i % 8 == 0:
+				y += 1
+
+	def __updateColors(self, bitNr, newState):
+		cb = self.cbs[bitNr]
+		pb = self.pbs[bitNr]
+		fr = self.__frames[bitNr]
+		palCb = cb.palette()
+		palPb = pb.palette()
+		palFr = fr.palette()
+		if newState == Qt.Checked:
+			cbColor = self.__cbActColor
+			pbColor = self.__pbActColor
+			frColor = self.__frActColor
+		else:
+			cbColor = self.__cbOrigColor
+			pbColor = self.__pbOrigColor
+			frColor = self.__frOrigColor
+		if palCb.color(QPalette.Base) != cbColor:
+			palCb.setColor(QPalette.Base, cbColor)
+			palPb.setColor(QPalette.Button, pbColor)
+			palFr.setColor(QPalette.Window, frColor)
+			cb.setPalette(palCb)
+			pb.setPalette(palPb)
+			fr.setPalette(palFr)
 
 	def __buttonUpdate(self):
-		for bitNr, pb in self.pbs.items():
+		for bitNr, pb in dictItems(self.pbs):
 			pressed = bool(pb.isDown())
 			if pressed == self.prevButtonStates[bitNr]:
 				continue
@@ -197,7 +266,7 @@ class BitDisplayWidget(AbstractDisplayWidget):
 
 	def get(self):
 		value = 0
-		for bitNr, cb in self.cbs.items():
+		for bitNr, cb in dictItems(self.cbs):
 			if cb.checkState() == Qt.Checked:
 				value |= (1 << bitNr)
 		return value
@@ -224,7 +293,9 @@ class NumberDisplayWidget(AbstractDisplayWidget):
 
 		self.line.valueChanged.connect(self.changed)
 
-		self.update()
+	def clear(self):
+		AbstractDisplayWidget.clear(self)
+		self.line.setText("invalid address")
 
 	def __convertValue(self, textValue):
 		try:
@@ -314,7 +385,9 @@ class RealDisplayWidget(AbstractDisplayWidget):
 
 		self.line.valueChanged.connect(self.changed)
 
-		self.update()
+	def clear(self):
+		AbstractDisplayWidget.clear(self)
+		self.line.setText("invalid address")
 
 	def __convertValue(self, textValue):
 		try:
@@ -353,6 +426,14 @@ class RealDisplayWidget(AbstractDisplayWidget):
 class State_Mem(StateWindow):
 	def __init__(self, client, addrSpace, parent=None):
 		StateWindow.__init__(self, client, parent)
+		if addrSpace == AbstractDisplayWidget.ADDRSPACE_E:
+			self.setWindowIcon(getIcon("inputs"))
+		elif addrSpace == AbstractDisplayWidget.ADDRSPACE_A:
+			self.setWindowIcon(getIcon("outputs"))
+		elif addrSpace == AbstractDisplayWidget.ADDRSPACE_M:
+			self.setWindowIcon(getIcon("flags"))
+		elif addrSpace == AbstractDisplayWidget.ADDRSPACE_DB:
+			self.setWindowIcon(getIcon("datablock"))
 
 		self.addrSpace = addrSpace
 
@@ -441,8 +522,7 @@ class State_Mem(StateWindow):
 		if fmt == "cb":
 			self.contentWidget = BitDisplayWidget(self,
 							      self.addrSpace,
-							      addr, width, db, self,
-							      displayPushButtons=True)
+							      addr, width, db, self)
 			self.contentLayout.addWidget(self.contentWidget)
 		elif fmt == "hex":
 			self.contentWidget = HexDisplayWidget(self,
@@ -468,14 +548,10 @@ class State_Mem(StateWindow):
 			assert(0)
 		self.contentWidget.changed.connect(self.__changed)
 		self.contentWidget.setEnabled(True)
-		self.update()
-		QTimer.singleShot(0, self.__finalizeRebuild)
+		self._updateSize(forceMinimum = True)
 
 		self.configChanged.emit(self)
 
-	def __finalizeRebuild(self):
-		self.resize(self.sizeHint())
-
 	def __changed(self):
 		if self.__changeBlocked or not self.contentWidget:
 			return
@@ -514,7 +590,9 @@ class State_Mem(StateWindow):
 
 		if memArea.flags & (memArea.FLG_ERR_READ | memArea.FLG_ERR_WRITE):
 			self.contentWidget.setEnabled(False)
+			self.contentWidget.clear()
 			return False
+		self.contentWidget.setEnabled(True)
 
 		with self.__changeBlocked:
 			addr = memArea.start
@@ -525,13 +603,14 @@ class State_Mem(StateWindow):
 					self.contentWidget.setByte(addr - thisStart,
 								   value)
 				addr += 1
-		self.update()
+		self._updateSize()
 		return True
 
 class State_LCD(StateWindow):
 	def __init__(self, client, parent=None):
 		StateWindow.__init__(self, client, parent)
 		self.setWindowTitle("LCD")
+		self.setWindowIcon(getIcon("lcd"))
 
 		self.displayedValue = 0
 
@@ -584,7 +663,7 @@ class State_LCD(StateWindow):
 		return self.endianCombo.itemData(index)
 
 	def rebuild(self):
-		self.update()
+		self._updateSize(forceMinimum = True)
 		self.configChanged.emit(self)
 
 	def __makeMemoryArea(self):
@@ -619,7 +698,7 @@ class State_LCD(StateWindow):
 					self.__setByte(addr - thisStart,
 						       value)
 				addr += 1
-		self.update()
+		self._updateSize()
 		return True
 
 	def __setByte(self, offset, value):
@@ -706,6 +785,8 @@ class _State_TimerCounter(StateWindow):
 		self.valueEdit = ValueLineEdit(self.__validateInput, self)
 		hbox.addWidget(self.valueEdit)
 		self.statusLabel = QLabel(self)
+		self.__statusLabelOrigColor = self.statusLabel.palette().color(QPalette.Window)
+		self.__statusLabelActColor = QColor(StateWindow.COLOR_ACTIVE)
 		hbox.addWidget(self.statusLabel)
 		self.resetButton = QPushButton("R", self)
 		hbox.addWidget(self.resetButton)
@@ -727,12 +808,23 @@ class _State_TimerCounter(StateWindow):
 
 	def __updateStatus(self):
 		self.statusLabel.setText("Q=%d" % self.displayedStatus)
+		pal = self.statusLabel.palette()
+		if self.displayedStatus == 0:
+			color = self.__statusLabelOrigColor
+			fill = False
+		else:
+			color = self.__statusLabelActColor
+			fill = True
+		if pal.color(QPalette.Window) != color:
+			pal.setColor(QPalette.Window, color)
+			self.statusLabel.setAutoFillBackground(fill)
+			self.statusLabel.setPalette(pal)
 
 	def rebuild(self):
 		self.valueEdit.clear()
 		self.displayedStatus = 0
 		self.__updateStatus()
-		self.update()
+		self._updateSize(forceMinimum = True)
 		self.configChanged.emit(self)
 
 	def __makeMemoryArea(self):
@@ -753,6 +845,7 @@ class _State_TimerCounter(StateWindow):
 
 		if memArea.flags & (memArea.FLG_ERR_READ | memArea.FLG_ERR_WRITE):
 			self.valueEdit.setEnabled(False)
+			self.valueEdit.clear()
 			return False
 		self.valueEdit.setEnabled(True)
 
@@ -773,7 +866,7 @@ class _State_TimerCounter(StateWindow):
 		with self.__changeBlocked:
 			self.valueEdit.setText(text)
 
-		self.update()
+		self._updateSize()
 		return True
 
 	def textToValue(self, text):
@@ -807,6 +900,7 @@ class State_Timer(_State_TimerCounter):
 					     MemoryArea.TYPE_T,
 					     parent)
 		self.setWindowTitle("Timer")
+		self.setWindowIcon(getIcon("timer"))
 
 		self.indexSpin.setPrefix("T ")
 
@@ -814,7 +908,7 @@ class State_Timer(_State_TimerCounter):
 		self.formatCombo.addItem("Hexadecimal", "hex")
 		self.formatCombo.addItem("S5Time", "s5t")
 
-		self.update()
+		self._updateSize(forceMinimum = True)
 
 	def textToValue(self, text):
 		fmt = self.formatCombo.itemData(self.formatCombo.currentIndex())
@@ -873,13 +967,14 @@ class State_Counter(_State_TimerCounter):
 					     MemoryArea.TYPE_Z,
 					     parent)
 		self.setWindowTitle("Counter")
+		self.setWindowIcon(getIcon("counter"))
 
 		self.indexSpin.setPrefix("C ")
 
 		self.formatCombo.addItem("BCD (counter)", "bcd")
 		self.formatCombo.addItem("Dual", "bin")
 
-		self.update()
+		self._updateSize(forceMinimum = True)
 
 	def textToValue(self, text):
 		fmt = self.formatCombo.itemData(self.formatCombo.currentIndex())
@@ -919,6 +1014,34 @@ class State_Counter(_State_TimerCounter):
 			assert(0)
 		return text
 
+class State_Blocks(StateWindow):
+	"""CPU online block view.
+	"""
+
+	# Signal: Open an item.
+	#	Argument: identHash
+	openItem = Signal(object)
+
+	def __init__(self, client, parent=None):
+		StateWindow.__init__(self, client, parent)
+		self.setWindowTitle("CPU online content (downloaded blocks)")
+		self.setWindowIcon(getIcon("plugin"))
+
+		self.__modelRef = client.getBlockTreeModelRef()
+
+		self.blockTree = BlockTreeView(self.__modelRef.obj, self)
+		self.layout().addWidget(self.blockTree, 0, 0)
+
+		self.setMinimumSize(450, 320)
+		self._updateSize()
+
+		self.blockTree.openItem.connect(self.openItem)
+
+	def closeEvent(self, ev):
+		self.__modelRef.destroy()
+		self.__modelRef = None
+		StateWindow.closeEvent(self, ev)
+
 class StateMdiArea(QMdiArea):
 	pass
 
