@@ -2,7 +2,7 @@
 #
 # AWL simulator - GUI edit widget
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,12 +20,12 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 from awlsim.gui.util import *
 from awlsim.gui.cpuwidget import *
 from awlsim.gui.sourcecodeedit import *
-from awlsim.gui.codevalidator import *
 
 
 class EditSubWidget(QWidget):
@@ -334,39 +334,51 @@ class CpuStatsContextMenu(QMenu):
 		self.closed.emit()
 
 class EditWidget(SourceCodeEdit):
+	"""AWL/STL edit widget."""
+
 	# Signal: Emitted, if the source code changed.
 	codeChanged = Signal()
 	# Signal: Emitted, if the visible source lines changed (e.g. scrolled)
 	visibleRangeChanged = Signal()
-	# Signal: editor<->Cpu source code match changed.
-	# Parameter 0: This EditWidget
-	# Parameter 1: Bool: True -> Editor code does match CPU code.
-	cpuCodeMatchChanged = Signal(SourceCodeEdit, bool)
 	# Signal: Keyboard focus in/out event.
 	focusChanged = Signal(bool)
 
-	__aniChars = ( ' ', '.', 'o', '0', 'O', '0', 'o', '.' )
+	# Generate the RUN animation
+	__runAniTemplate = ("   x   ",
+			    "  -x-  ", " --x-- ", "---x---",
+			    "--=x=--", "-==x==-", "===x===",
+			    "==*x*==", "=**x**=", "***x***")
+	__runAniTemplate = __runAniTemplate + __runAniTemplate[1:-1][::-1]
+	__runAni = tuple(c.replace("x", " CPU running ")
+			 for c in __runAniTemplate)
+	__runAniNoDown = tuple(c.replace("x", " source NOT DOWNLOADED to CPU ")
+			       for c in __runAniTemplate)
 
-	def __init__(self, parent=None):
+	def __init__(self, parent=None,
+		     readOnly=False,
+		     withHeader=True, withCpuStats=True):
 		SourceCodeEdit.__init__(self, parent)
 
-		self.__validatorResults = []
-		self.__validatorTimer = QTimer(self)
-		self.__validatorTimer.setSingleShot(True)
-		self.__validatorTimer.timeout.connect(self.__checkValidator)
+		self.setReadOnly(readOnly)
 
-		self.__aniTimer = QTimer(self)
-		self.__aniTimer.setSingleShot(False)
-		self.__aniTimer.timeout.connect(self.__animation)
+		self.__runAniTimer = QTimer(self)
+		self.__runAniTimer.setSingleShot(False)
+		self.__runAniTimer.timeout.connect(self.__runAnimation)
 
 		self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
 		self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
 		self.setLineWrapMode(SourceCodeEdit.NoWrap)
 		self.setTabStopWidth(self.tabStopWidth() // 2)
 
-		self.headerWidget = HeaderSubWidget(self)
+		if withHeader:
+			self.headerWidget = HeaderSubWidget(self)
+		else:
+			self.headerWidget = None
 		self.lineNumWidget = LineNumSubWidget(self)
-		self.cpuStatsWidget = CpuStatsSubWidget(self)
+		if withCpuStats:
+			self.cpuStatsWidget = CpuStatsSubWidget(self)
+		else:
+			self.cpuStatsWidget = None
 
 		self.__updateFont(getDefaultFixedFont())
 
@@ -378,13 +390,15 @@ class EditWidget(SourceCodeEdit):
 		self.__nextHdrUpdate = 0
 		self.__hdrAniStat = 0
 
-		self.__cpuStatsMenu = CpuStatsContextMenu(self)
-		self.__cpuStatsMask = 0
+		if withCpuStats:
+			self.__cpuStatsMenu = CpuStatsContextMenu(self)
+			self.__cpuStatsMask = 0
 		self.enableCpuStats(enabled=False, force=True)
-		self.setCpuStatsMask(CpuStatsEntry.SHOW_VKE |\
-				     CpuStatsEntry.SHOW_STA |\
-				     CpuStatsEntry.SHOW_ACCU1 |\
-				     CpuStatsEntry.SHOW_ACCU2)
+		if withCpuStats:
+			self.setCpuStatsMask(CpuStatsEntry.SHOW_VKE |\
+					     CpuStatsEntry.SHOW_STA |\
+					     CpuStatsEntry.SHOW_ACCU1 |\
+					     CpuStatsEntry.SHOW_ACCU2)
 		self.resetCpuStats(True)
 
 		self.__textChangeBlocked = Blocker()
@@ -392,26 +406,23 @@ class EditWidget(SourceCodeEdit):
 
 		self.blockCountChanged.connect(self.__updateMargins)
 		self.updateRequest.connect(self.__updateExtraWidgets)
-		self.headerWidget.needRepaint.connect(self.__repaintHeaderWidget)
 		self.lineNumWidget.needRepaint.connect(self.__repaintLineNumWidget)
-		self.cpuStatsWidget.needRepaint.connect(self.__repaintCpuStatsWidget)
-		self.headerWidget.wasScrolled.connect(self.__forwardWheelEvent)
-		self.headerWidget.contextMenuReq.connect(self.__cpuStatsContextMenuPopup)
 		self.lineNumWidget.wasScrolled.connect(self.__forwardWheelEvent)
-		self.cpuStatsWidget.wasScrolled.connect(self.__forwardWheelEvent)
-		self.cpuStatsWidget.contextMenuReq.connect(self.__cpuStatsContextMenuPopup)
-		self.__cpuStatsMenu.closed.connect(self.__cpuStatsContextMenuClosed)
+		if self.headerWidget:
+			self.headerWidget.needRepaint.connect(self.__repaintHeaderWidget)
+			self.headerWidget.wasScrolled.connect(self.__forwardWheelEvent)
+			self.headerWidget.contextMenuReq.connect(self.__cpuStatsContextMenuPopup)
+		if self.cpuStatsWidget:
+			self.cpuStatsWidget.needRepaint.connect(self.__repaintCpuStatsWidget)
+			self.cpuStatsWidget.wasScrolled.connect(self.__forwardWheelEvent)
+			self.cpuStatsWidget.contextMenuReq.connect(self.__cpuStatsContextMenuPopup)
+			self.__cpuStatsMenu.closed.connect(self.__cpuStatsContextMenuClosed)
+
+	def getSourceId(self):
+		return self.__source.identHash
 
 	def shutdown(self):
-		while self.__validatorResults:
-			self.__checkValidator(rearm=False)
-			QApplication.processEvents(QEventLoop.ExcludeUserInputEvents)
-		self.__validatorTimer.stop()
-		self.__validatorTimer = None
-
-		# Make sure all QTimer objects are collected.
-		# This avoids crashes within Qt.
-		garbageCollector.collect()
+		pass
 
 	def setSource(self, source):
 		with self.__textChangeBlocked:
@@ -419,13 +430,13 @@ class EditWidget(SourceCodeEdit):
 			self.__source.sourceBytes = b""
 			sourceBytes = source.sourceBytes
 			try:
-				sourceText = sourceBytes.decode(AwlParser.TEXT_ENCODING,
+				sourceText = sourceBytes.decode(AwlSource.ENCODING,
 								errors="strict")
 			except UnicodeError:
 				MessageBox.error(self, "The AWL/STL code contains "
 					"non-%s-characters. These were ignored and stripped "
-					"from the code." % AwlParser.TEXT_ENCODING)
-				sourceText = sourceBytes.decode(AwlParser.TEXT_ENCODING,
+					"from the code." % AwlSource.ENCODING)
+				sourceText = sourceBytes.decode(AwlSource.ENCODING,
 								errors="ignore")
 			self.setPlainText(sourceText)
 			self.resetCpuStats()
@@ -434,17 +445,17 @@ class EditWidget(SourceCodeEdit):
 	def __updateSource(self):
 		sourceText = self.toPlainText()
 		# Convert to DOS-style line endings
-		sourceText = "\r\n".join(sourceText.splitlines()) + "\r\n"
+		sourceText = toDosEol(sourceText)
 		# Convert to binary
 		try:
-			sourceBytes = sourceText.encode(AwlParser.TEXT_ENCODING,
+			sourceBytes = sourceText.encode(AwlSource.ENCODING,
 							errors="strict")
 			self.__source.sourceBytes = sourceBytes
 		except UnicodeError:
 			MessageBox.error(self, "The AWL/STL code contains "
 				"non-%s-characters. These were ignored and stripped "
-				"from the code." % AwlParser.TEXT_ENCODING)
-			sourceBytes = sourceText.encode(AwlParser.TEXT_ENCODING,
+				"from the code." % AwlSource.ENCODING)
+			sourceBytes = sourceText.encode(AwlSource.ENCODING,
 							errors="ignore")
 			self.__source.sourceBytes = sourceBytes
 			self.setSource(self.__source)
@@ -468,18 +479,19 @@ class EditWidget(SourceCodeEdit):
 
 	def runStateChanged(self, newState):
 		self.__runState = newState
-		if newState.state == RunState.STATE_LOAD:
+		if newState == RunState.STATE_LOAD:
 			self.resetCpuStats()
 			self.__setSourceMatchesCpuSource(True)
-		if newState.state == RunState.STATE_RUN:
+		if newState == RunState.STATE_RUN:
 			self.__setSourceMatchesCpuSource(
 				self.__sourceMatchesCpuSource, force = True)
-			self.__aniTimer.start(200)
+			self.__runAniTimer.start(200)
 		else:
-			self.__aniTimer.stop()
+			self.__runAniTimer.stop()
 		if self.__cpuStatsEnabled:
 			self.cpuStatsWidget.update()
-		self.headerWidget.update()
+		if self.headerWidget:
+			self.headerWidget.update()
 
 	def __eachVisibleLine(self):
 		block = self.firstVisibleBlock()
@@ -520,18 +532,22 @@ class EditWidget(SourceCodeEdit):
 		# Set the font for other window elements
 		self.setFont(font)
 		self.update()
-		self.headerWidget.setFont(font)
-		self.headerWidget.update()
+		if self.headerWidget:
+			self.headerWidget.setFont(font)
+			self.headerWidget.update()
 		self.lineNumWidget.setFont(font)
 		self.lineNumWidget.update()
-		self.cpuStatsWidget.setFont(font)
-		self.cpuStatsWidget.update()
+		if self.cpuStatsWidget:
+			self.cpuStatsWidget.setFont(font)
+			self.cpuStatsWidget.update()
 
 		# Cache metrics
 		self.__charHeight = self.fontMetrics().height()
 		self.setTabStopWidth(self.fontMetrics().width("X") * 8)
 
 	def enableCpuStats(self, enabled=True, force=False):
+		if not self.cpuStatsWidget:
+			enabled = False
 		if force or enabled != self.__cpuStatsEnabled:
 			self.__cpuStatsEnabled = enabled
 			self.__updateMargins()
@@ -559,14 +575,16 @@ class EditWidget(SourceCodeEdit):
 		self.__cpuStatsUpdate = 1
 		self.__cpuStatsStamp = 0
 		self.__updateMargins()
-		self.headerWidget.update()
-		self.cpuStatsWidget.update()
+		if self.headerWidget:
+			self.headerWidget.update()
+		if self.cpuStatsWidget:
+			self.cpuStatsWidget.update()
 
 	def updateCpuStats_afterInsn(self, insnDumpMsg):
 		# insnDumpMsg => AwlSimMessage_INSNDUMP instance
 		if not self.__cpuStatsEnabled:
 			return
-		if insnDumpMsg.sourceId != self.__source.identHash:
+		if insnDumpMsg.sourceId != self.getSourceId():
 			# Discard old messages that were still in the queue.
 			return
 		# Save the instruction dump
@@ -583,33 +601,34 @@ class EditWidget(SourceCodeEdit):
 		# First instruction in cycle?
 		if insnDumpMsg.serial == 0:
 			# Update the 'obsolete'-flag based on the timestamp
-			for ent in self.__lineCpuStats.values():
+			for ent in dictValues(self.__lineCpuStats):
 				ent.obsolete = (ent.stamp != self.__cpuStatsStamp)
 			# Advance the timestamp
 			self.__cpuStatsStamp += 1
 
 	def __pruneInvisibleCpuStats(self):
-		if self.__runState.state == RunState.STATE_OFFLINE:
+		if self.__runState == RunState.STATE_OFFLINE:
 			return
 		firstLine, lastLine = self.getVisibleLineRange()
-		for line, stats in self.__lineCpuStats.items():
+		for line, stats in dictItems(self.__lineCpuStats):
 			if line < firstLine or line > lastLine:
 				stats.pruned = True
 				stats.obsolete = True
 
-	def __animation(self):
-		self.__hdrAniStat = (self.__hdrAniStat + 1) %\
-				    len(self.__aniChars)
-		self.headerWidget.update()
+	def __runAnimation(self):
+		if self.headerWidget:
+			self.__hdrAniStat = (self.__hdrAniStat + 1) %\
+					    len(self.__runAni)
+			self.headerWidget.update()
 
 	def __setSourceMatchesCpuSource(self, sourceIsOnCpu, force=False):
 		if sourceIsOnCpu != self.__sourceMatchesCpuSource or force:
 			self.__sourceMatchesCpuSource = sourceIsOnCpu
-			self.headerWidget.update()
-			self.cpuCodeMatchChanged.emit(self, sourceIsOnCpu)
+			if self.headerWidget:
+				self.headerWidget.update()
 
 	def handleIdentsMsg(self, identsMsg):
-		if self.__runState.state == RunState.STATE_RUN:
+		if self.__runState == RunState.STATE_RUN:
 			cpuHashes = [ s.identHash for s in identsMsg.awlSources ]
 			self.__setSourceMatchesCpuSource(
 				self.getSource().identHash in cpuHashes)
@@ -635,7 +654,9 @@ class EditWidget(SourceCodeEdit):
 		return 5 + 5 + metr.width(self.cpuStatsWidget.getBanner(self.__cpuStatsMask).replace(" ", "_"))
 
 	def headerHeight(self):
-		return 5 + 5 + self.__charHeight
+		if self.headerWidget:
+			return 5 + 5 + self.__charHeight
+		return 0
 
 	def __updateMargins(self):
 		self.setViewportMargins(self.lineNumWidgetWidth(),
@@ -651,12 +672,13 @@ class EditWidget(SourceCodeEdit):
 		return sh
 
 	def __updateHeaderWidgetGeo(self):
-		cont = self.contentsRect()
-		rect = QRect(cont.left(),
-			     cont.top(),
-			     cont.width(),
-			     self.headerHeight())
-		self.headerWidget.setGeometry(rect)
+		if self.headerWidget:
+			cont = self.contentsRect()
+			rect = QRect(cont.left(),
+				     cont.top(),
+				     cont.width(),
+				     self.headerHeight())
+			self.headerWidget.setGeometry(rect)
 
 	def __updateLineNumWidgetGeo(self):
 		cont = self.contentsRect()
@@ -667,6 +689,8 @@ class EditWidget(SourceCodeEdit):
 		self.lineNumWidget.setGeometry(rect)
 
 	def __updateCpuStatsWidgetGeo(self):
+		if not self.cpuStatsWidget:
+			return
 		if self.__cpuStatsEnabled:
 			vp, cont = self.viewport(), self.contentsRect()
 			rect = QRect(vp.width() + self.lineNumWidgetWidth(),
@@ -685,21 +709,25 @@ class EditWidget(SourceCodeEdit):
 
 	def __updateExtraWidgets(self, rect, dy):
 		if dy:
-			self.headerWidget.scroll(0, dy)
+			if self.headerWidget:
+				self.headerWidget.scroll(0, dy)
 			self.lineNumWidget.scroll(0, dy)
-			self.cpuStatsWidget.scroll(0, dy)
+			if self.cpuStatsWidget:
+				self.cpuStatsWidget.scroll(0, dy)
 			self.__pruneInvisibleCpuStats()
 			self.visibleRangeChanged.emit()
 			return
-		self.headerWidget.update(0, rect.y(),
-			self.headerWidget.width(),
-			rect.height())
+		if self.headerWidget:
+			self.headerWidget.update(0, rect.y(),
+				self.headerWidget.width(),
+				rect.height())
 		self.lineNumWidget.update(0, rect.y(),
 			self.lineNumWidget.width(),
 			rect.height())
-		self.cpuStatsWidget.update(0, rect.y(),
-			self.cpuStatsWidget.width(),
-			rect.height())
+		if self.cpuStatsWidget:
+			self.cpuStatsWidget.update(0, rect.y(),
+				self.cpuStatsWidget.width(),
+				rect.height())
 		if rect.contains(self.viewport().rect()):
 			self.__updateMargins()
 
@@ -723,8 +751,14 @@ class EditWidget(SourceCodeEdit):
 	}
 
 	def __repaintHeaderWidget(self, ev):
+		if not self.headerWidget:
+			return
 		p = self.headerWidget.getPainter()
-		p.fillRect(ev.rect(), Qt.lightGray)
+		if not self.__sourceMatchesCpuSource and\
+		   self.__runState == RunState.STATE_RUN:
+			p.fillRect(ev.rect(), getErrorColor())
+		else:
+			p.fillRect(ev.rect(), Qt.lightGray)
 
 		if self.__cpuStatsEnabled:
 			# Map the CPU-stats start point to header widget
@@ -737,26 +771,32 @@ class EditWidget(SourceCodeEdit):
 			textMaxPixels = self.headerWidget.width()
 
 		if self.__runState.spawned:
-			runText = "[SIM]: "
+			runText = [ "[SIM]: ", ]
 		else:
-			runText = "[%s:%d]: " %(\
+			runText = [ "[%s:%d%s]: " %(\
 				self.__runState.host,
-				self.__runState.port)
-		if self.__runState.state == RunState.STATE_RUN:
-			runText += self.__aniChars[self.__hdrAniStat]
-			if not self.__sourceMatchesCpuSource:
-				runText += " [NOT DOWNLOADED to CPU]"
+				self.__runState.port,
+				" via SSH" if\
+				self.__runState.haveTunnel else ""), ]
+		if self.__runState == RunState.STATE_RUN:
+			if self.__sourceMatchesCpuSource:
+				runText.append(self.__runAni[self.__hdrAniStat])
+			else:
+				runText.append(self.__runAniNoDown[self.__hdrAniStat])
 		else:
-			runText += self.__runStateToText[self.__runState.state]
+			runText.append(self.__runStateToText[self.__runState.state])
 
 		# Limit the text length to the available space.
 		metr = self.headerWidget.fontMetrics()
 		maxNrChars = textMaxPixels // metr.width('_')
+		runText = "".join(runText)
 		if len(runText) > maxNrChars:
 			runText = runText[ : max(0, maxNrChars - 3)]
 			runText += "..."
 			runText = runText[ : maxNrChars]
 
+		p.setPen(QPen(QColor("black")))
+
 		p.drawText(5, 5,
 			   metr.width(runText.replace(" ", "_")),
 			   self.headerWidget.height(),
@@ -787,6 +827,8 @@ class EditWidget(SourceCodeEdit):
 				   str(lineNr))
 
 	def __repaintCpuStatsWidget(self, ev):
+		if not self.cpuStatsWidget:
+			return
 		p = self.cpuStatsWidget.getPainter()
 		rect = ev.rect()
 		p.fillRect(rect, Qt.lightGray)
@@ -812,26 +854,43 @@ class EditWidget(SourceCodeEdit):
 		self.__needSourceUpdate = True
 		self.codeChanged.emit()
 		self.resetCpuStats()
-		if self.__runState.state != RunState.STATE_RUN:
+		if self.__runState != RunState.STATE_RUN:
 			# The CPU is not in RUN state.
 			# We don't make a big deal out of code mismatch, even
 			# if the code most likely _does_ mismatch after this edit.
 			self.__setSourceMatchesCpuSource(True)
 
-	# Validation callback. Overridden subclass method.
-	def validateText(self, text, currentLineNr):
-		result = AwlValidator.get().enqueue(text)
-		self.__validatorResults.append(result)
-		if not self.__validatorTimer.isActive():
-			self.__validatorTimer.start(50)
+	def handleValidationResult(self, exception):
+		if exception and exception.getSourceId() == self.getSourceId():
+			lineNr = exception.getLineNr()
+			if lineNr:
+				self.setErraticLine(lineNr - 1, str(exception))
+				return
+		self.setErraticLine(None)
 
-	def __checkValidator(self, rearm=True):
-		if not self.__validatorResults or\
-		   not self.__validatorResults[0].ready():
-			return
-		result = self.__validatorResults.pop(0)
+class EditDialog(QDialog):
+	"""AWL/STL edit dialog."""
 
-		self.setErraticLines(result.getErrLines())
+	def __init__(self, parent=None,
+		     readOnly=False,
+		     withHeader=True, withCpuStats=True,
+		     okButton=False, cancelButton=False):
+		QDialog.__init__(self, parent)
+		self.setLayout(QGridLayout())
 
-		if self.__validatorResults and rearm:
-			self.__validatorTimer.start(25)
+		self.edit = EditWidget(self, readOnly=readOnly,
+				       withHeader=withHeader,
+				       withCpuStats=withCpuStats)
+		self.layout().addWidget(self.edit, 0, 0, 1, 2)
+
+		if okButton:
+			self.__okButton = QPushButton("&Ok", self)
+			self.layout().addWidget(self.__okButton, 1, 0)
+			self.__okButton.released.connect(self.accept)
+
+		if cancelButton:
+			self.__cancelButton = QPushButton("&Cancel", self)
+			self.layout().addWidget(self.__cancelButton, 1, 1)
+			self.__cancelButton.released.connect(self.reject)
+
+		self.resize(1000, 550)
