@@ -20,6 +20,7 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 from awlsim.gui.util import *
@@ -53,10 +54,8 @@ class GenericActionWidget(QWidget):
 		for ftype in (BlockInterfaceField.FTYPE_IN,
 			      BlockInterfaceField.FTYPE_OUT,
 			      BlockInterfaceField.FTYPE_INOUT):
-			try:
+			with contextlib.suppress(KeyError):
 				fields.extend(interfaceFields[ftype])
-			except KeyError:
-				pass
 		ret = [ "CALL %s%s%s" %\
 			(targetName,
 			 ", DB ..." if needDB else "",
@@ -89,8 +88,7 @@ class GenericActionWidget(QWidget):
 			desc.append("  " + fname)
 			for field in fields:
 				field.fieldType = ftype
-				desc.append("    %s : %s;" % (field.name,
-							      str(field.dataType)))
+				desc.append("    %s" % field.varDeclString())
 		return "\n".join(desc)
 
 	def defaultPaste(self):
@@ -105,18 +103,18 @@ class SysActionWidget(GenericActionWidget):
 
 		self.desc = QLabel(self)
 		self.desc.setFont(getDefaultFixedFont())
-		self.layout().addWidget(self.desc, 0, 0)
+		self.layout().addWidget(self.desc, 0, 0, 1, 2)
 
 		self.layout().setRowStretch(1, 1)
 
 		label = QLabel("Paste at cursor position:", self)
-		self.layout().addWidget(label, 2, 0)
+		self.layout().addWidget(label, 2, 0, 1, 2)
 
 		self.pasteCallSymButton = QPushButton(self)
 		self.layout().addWidget(self.pasteCallSymButton, 3, 0)
 
 		self.pasteCallButton = QPushButton(self)
-		self.layout().addWidget(self.pasteCallButton, 4, 0)
+		self.layout().addWidget(self.pasteCallButton, 3, 1)
 
 		self.pasteCallSymButton.released.connect(self.__pasteCallSym)
 		self.pasteCallButton.released.connect(self.__pasteCall)
@@ -137,14 +135,14 @@ class SysActionWidget(GenericActionWidget):
 	def __pasteCall(self):
 		blockNumber, symbolName, blockDesc = self.systemBlockCls.name
 		self._pasteCallGeneric("%s %d" % (self.blockPrefix, blockNumber),
-				       self.systemBlockCls.isFB,
+				       self.systemBlockCls._isFB,
 				       self.systemBlockCls.interfaceFields)
 		self.finish.emit()
 
 	def __pasteCallSym(self):
 		blockNumber, symbolName, blockDesc = self.systemBlockCls.name
 		self._pasteCallGeneric('"%s"' % symbolName,
-				       self.systemBlockCls.isFB,
+				       self.systemBlockCls._isFB,
 				       self.systemBlockCls.interfaceFields)
 		self.addSymbol.emit(symbolName,
 				    "%s %s" % (self.blockPrefix, blockNumber),
@@ -163,24 +161,24 @@ class LibActionWidget(GenericActionWidget):
 
 		self.desc = QLabel(self)
 		self.desc.setFont(getDefaultFixedFont())
-		self.layout().addWidget(self.desc, 0, 0)
+		self.layout().addWidget(self.desc, 0, 0, 1, 2)
 
 		self.layout().setRowStretch(1, 1)
 
 		label = QLabel("Paste at cursor position:", self)
-		self.layout().addWidget(label, 2, 0)
+		self.layout().addWidget(label, 2, 0, 1, 2)
 
 		self.pasteCallSymButton = QPushButton(self)
 		self.layout().addWidget(self.pasteCallSymButton, 3, 0)
 
 		self.pasteCallButton = QPushButton(self)
-		self.layout().addWidget(self.pasteCallButton, 4, 0)
+		self.layout().addWidget(self.pasteCallButton, 3, 1)
 
 		self.pasteCodeSymButton = QPushButton(self)
-		self.layout().addWidget(self.pasteCodeSymButton, 5, 0)
+		self.layout().addWidget(self.pasteCodeSymButton, 4, 0)
 
 		self.pasteCodeButton = QPushButton(self)
-		self.layout().addWidget(self.pasteCodeButton, 6, 0)
+		self.layout().addWidget(self.pasteCodeButton, 4, 1)
 
 		self.pasteCallSymButton.released.connect(self.__pasteCallSym)
 		self.pasteCallButton.released.connect(self.__pasteCall)
@@ -190,8 +188,8 @@ class LibActionWidget(GenericActionWidget):
 	def updateData(self, libEntryCls):
 		self.libEntryCls = libEntryCls
 
-		prefix = "FC" if libEntryCls.isFC else "FB"
-		typeStr = "FUNCTION" if libEntryCls.isFC else "FUNCTION_BLOCK"
+		prefix = "FC" if libEntryCls._isFC else "FB"
+		typeStr = "FUNCTION" if libEntryCls._isFC else "FUNCTION_BLOCK"
 		self.blockName = "%s %d" % (prefix, libEntryCls.staticIndex)
 
 		desc = self._blockToInterfaceText(self.blockName,
@@ -214,7 +212,7 @@ class LibActionWidget(GenericActionWidget):
 			"Warning: It is not recommended to paste library "
 			"code into the project sources. You should instead "
 			"just import the library (via library selection table) "
-			"and CALL the imported function.\n"
+			"and CALL the imported function.\n\n"
 			"See the 'CALL \"%s\"' or 'CALL %s' buttons.\n\n"
 			"Do you want to paste the code nevertheless?" % (
 			self.libEntryCls.symbolName, self.blockName),
@@ -240,14 +238,14 @@ class LibActionWidget(GenericActionWidget):
 
 	def __pasteCall(self):
 		self._pasteCallGeneric(self.blockName,
-				       self.libEntryCls.isFB,
+				       self.libEntryCls._isFB,
 				       self.libEntryCls.interfaceFields)
 		self.addLibrary.emit(self.libEntryCls().makeSelection())
 		self.finish.emit()
 
 	def __pasteCallSym(self):
 		self._pasteCallGeneric('"%s"' % self.libEntryCls.symbolName,
-				       self.libEntryCls.isFB,
+				       self.libEntryCls._isFB,
 				       self.libEntryCls.interfaceFields)
 		self.addSymbol.emit(self.libEntryCls.symbolName,
 				    self.blockName,
@@ -324,9 +322,9 @@ class LibraryDialog(QDialog):
 
 		self.libList.setCurrentRow(0)
 
-		self.libList.setMinimumWidth(200)
-		self.libElemList.setMinimumWidth(380)
-		self.iconLabel.setMinimumWidth(220)
+		self.libList.setMinimumWidth(190)
+		self.libElemList.setMinimumWidth(350)
+		self.iconLabel.setMinimumWidth(400)
 		self.sysAction.setMinimumWidth(self.iconLabel.minimumWidth())
 		self.libAction.setMinimumWidth(self.iconLabel.minimumWidth())
 		self.resize(self.size().width(), 360)
@@ -349,11 +347,12 @@ class LibraryDialog(QDialog):
 
 	def __addSystemBlockTable(self, prefix, table):
 		entries = []
-		for blockCls in sorted(table.values(), key=lambda c: c.name[0]):
+		for blockCls in sorted(dictValues(table), key=lambda c: c.name[0]):
 			if blockCls.broken:
 				continue
 			number, name, desc = blockCls.name
-			if number < 0 and not self.project.getExtInsnsEn():
+			conf = self.project.getCpuConf()
+			if number < 0 and not conf.extInsnsEn:
 				continue
 
 			absName = "%s %d" % (prefix, number)
@@ -368,10 +367,11 @@ class LibraryDialog(QDialog):
 
 	def __addLibraryTable(self, lib):
 		entries = []
-		for i, libCls in enumerate(lib.entries()):
+		for i, libCls in enumerate(sorted(lib.entries(),
+						  key=lambda c: c.staticIndex)):
 			if libCls.broken:
 				continue
-			absName = "%s %d" % ("FC" if libCls.isFC else "FB",\
+			absName = "%s %d" % ("FC" if libCls._isFC else "FB",\
 					     libCls.staticIndex)
 			symName = '"%s"' % libCls.symbolName
 			if libCls.description:
