@@ -20,6 +20,7 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 from awlsim.gui.util import *
@@ -52,6 +53,14 @@ class LibTableModel(QAbstractTableModel):
 			del self.libSelections[row]
 			self.endResetModel()
 			self.contentChanged.emit()
+			return True
+		return False
+
+	def deleteEntries(self, rows):
+		offset = 0
+		for row in sorted(rows):
+			if self.deleteEntry(row + offset):
+				offset -= 1
 
 	def addEntry(self, libSelection):
 		for sel in self.libSelections:
@@ -60,7 +69,7 @@ class LibTableModel(QAbstractTableModel):
 			   sel.getEntryIndex() == libSelection.getEntryIndex():
 				return
 		self.libSelections.append(libSelection)
-		self.rowsInserted.emit(None,
+		self.rowsInserted.emit(QModelIndex(),
 			len(self.libSelections),
 			len(self.libSelections))
 		self.contentChanged.emit()
@@ -111,16 +120,20 @@ class LibTableModel(QAbstractTableModel):
 				)
 			else:
 				assert(0)
-		elif role == Qt.BackgroundRole:
+		elif role in {Qt.BackgroundRole,
+			      Qt.ForegroundRole}:
 			if row < len(self.libSelections):
 				sel = self.libSelections[row]
 				if not sel.isValid():
-					return QBrush(QColor("red"))
+					if role == Qt.BackgroundRole:
+						return QBrush(QColor("red"))
+					return QBrush(QColor("black"))
 				try:
 					AwlLib.getEntryBySelection(sel)
 				except AwlSimError as e:
-					return QBrush(QColor("orange"))
-			return QBrush(QColor("white"))
+					if role == Qt.BackgroundRole:
+						return QBrush(QColor("orange"))
+					return QBrush(QColor("black"))
 		elif role in (Qt.ToolTipRole, Qt.WhatsThisRole):
 			return (
 				# Library name
@@ -173,7 +186,7 @@ class LibTableModel(QAbstractTableModel):
 			if row >= len(self.libSelections):
 				sel = AwlLibEntrySelection()
 				self.libSelections.append(sel)
-				self.rowsInserted.emit(None,
+				self.rowsInserted.emit(QModelIndex(),
 					len(self.libSelections),
 					len(self.libSelections))
 			else:
@@ -229,7 +242,10 @@ class LibTableView(QTableView):
 			model = LibTableModel()
 		self.setModel(model)
 
-		self.verticalHeader().setMovable(True)
+		if isQt4:
+			self.verticalHeader().setMovable(True)
+		else:
+			self.verticalHeader().setSectionsMovable(True)
 		self.verticalHeader().sectionMoved.connect(self.__rowMoved)
 
 		self.pressed.connect(self.__handleMousePress)
@@ -263,12 +279,12 @@ class LibTableView(QTableView):
 		QTableView.focusOutEvent(self, ev)
 		self.focusChanged.emit(False)
 
-	def deleteEntry(self, index=None):
-		if not index:
-			index = self.currentIndex()
-		if not index:
-			return
-		self.model().deleteEntry(index.row())
+	def deleteEntries(self, rows=None):
+		if rows is None:
+			rows = set()
+			for index in self.selectedIndexes():
+				rows.add(index.row())
+		self.model().deleteEntries(rows)
 
 	def addEntry(self, libSelection):
 		self.model().addEntry(libSelection)
@@ -279,7 +295,12 @@ class LibTableView(QTableView):
 			pass#TODO context menu
 
 	def keyPressEvent(self, ev):
+		if ev.matches(QKeySequence.Delete):
+			self.deleteEntries()
+			ev.accept()
+			return
+
 		QTableView.keyPressEvent(self, ev)
 
-		if ev.key() == Qt.Key_Delete:
-			self.deleteEntry()
+	def handleValidationResult(self, exception):
+		pass
