@@ -2,7 +2,7 @@
 #
 # AWL simulator - Generic source code edit widget
 #
-# Copyright 2014 Michael Buesch <m@bues.ch>
+# Copyright 2014-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
+from awlsim.gui.finddialog import *
 from awlsim.gui.util import *
 
 
@@ -30,9 +32,28 @@ class SourceCodeEdit(QPlainTextEdit):
 	#         If the parameter is True, increase font size.
 	resizeFont = Signal(bool)
 
+	# Signal: Validation request.
+	#	  A code validation should take place.
+	#	  In case of validation failure, call setErraticLine().
+	#	  The parameter is self.
+	validateDocument = Signal(QObject)
+
+	__findDialog = None
+
+	@classmethod
+	def initFindDialog(cls, parent):
+		cls.__findDialog = FindReplaceDialog(parent)
+
+	@classmethod
+	def getFindDialog(cls):
+		return cls.__findDialog
+
 	def __init__(self, parent=None):
 		QPlainTextEdit.__init__(self, parent)
 
+		self.__errLineBrushBg = QBrush(getErrorColor())
+		self.__errLineBrushFg = QBrush(QColor("black"))
+
 		self.__wheelSteps = 0.0
 
 		self.__undoIsAvailable = False
@@ -46,9 +67,11 @@ class SourceCodeEdit(QPlainTextEdit):
 		self.enableAutoIndent()
 		self.enablePasteIndent()
 		self.__validateEn = True
-		self.__prevErrLines = ()
+		self.__errLineNr = None
+		self.__errLineMsg = None
 
 		self.cursorPositionChanged.connect(self.__handleCursorChange)
+		self.textChanged.connect(self.__handleTextChange)
 		self.undoAvailable.connect(self.__handleUndoAvailableChange)
 		self.redoAvailable.connect(self.__handleRedoAvailableChange)
 		self.copyAvailable.connect(self.__handleCopyAvailableChange)
@@ -79,7 +102,9 @@ class SourceCodeEdit(QPlainTextEdit):
 
 	def enableValidation(self, enable=True):
 		self.__validateEn = enable
-		self.setErraticLines(())
+		if not enable:
+			QToolTip.hideText()
+		self.setErraticLine(None)
 		self.__validate()
 
 	def __getLineIndent(self, cursor):
@@ -124,6 +149,15 @@ class SourceCodeEdit(QPlainTextEdit):
 			cursor.deleteChar()
 
 	def keyPressEvent(self, ev):
+		if ev.matches(QKeySequence.Find):
+			self.findText()
+			ev.accept()
+			return
+		elif ev.matches(QKeySequence.Replace):
+			self.findReplaceText()
+			ev.accept()
+			return
+
 		QPlainTextEdit.keyPressEvent(self, ev)
 
 		if ev.key() in (Qt.Key_Return, Qt.Key_Enter):
@@ -134,7 +168,10 @@ class SourceCodeEdit(QPlainTextEdit):
 	def wheelEvent(self, ev):
 		if ev.modifiers() & Qt.ControlModifier:
 			# Ctrl + Scroll-wheel: Font resizing
-			numDegrees = ev.delta() / 8
+			if isQt4:
+				numDegrees = ev.delta() / 8
+			else:
+				numDegrees = ev.angleDelta().y() / 8
 			numSteps = numDegrees / 15
 			self.__wheelSteps += numSteps
 			if self.__wheelSteps >= 1.0:
@@ -145,9 +182,11 @@ class SourceCodeEdit(QPlainTextEdit):
 				while self.__wheelSteps <= -1.0:
 					self.resizeFont.emit(True)
 					self.__wheelSteps += 1.0
+			ev.accept()
 			return
 		else:
 			self.__wheelSteps = 0.0
+			ev.ignore()
 
 		QPlainTextEdit.wheelEvent(self, ev)
 
@@ -183,7 +222,7 @@ class SourceCodeEdit(QPlainTextEdit):
 		if columnNr != self.__prevColumnNr:
 			self.__columnChange = True
 
-		if lineNr in self.__prevErrLines or\
+		if lineNr == self.__errLineNr or\
 		   (lineNr != self.__prevLineNr and\
 		    self.__columnChange):
 			self.__validate()
@@ -192,7 +231,14 @@ class SourceCodeEdit(QPlainTextEdit):
 		self.__prevColumnNr = columnNr
 		self.__prevLineNr = lineNr
 
+	def __handleTextChange(self):
+		QToolTip.hideText()
+		if self.__errLineNr is not None:
+			self.__validate()
+
 	def setPlainText(self, text):
+		self.__errLineNr = None
+		self.__errLineMsg = None
 		QPlainTextEdit.setPlainText(self, text)
 		self.__validate()
 
@@ -212,34 +258,88 @@ class SourceCodeEdit(QPlainTextEdit):
 		cursor.select(QTextCursor.LineUnderCursor)
 		return cursor
 
-	__errLineBrush = QBrush(QColor("#FFC0C0"))
-
 	def __validate(self):
 		if not self.__validateEn:
 			return
 
 		# Validate the current document
-		cursor = self.textCursor()
-		self.validateText(self.toPlainText(),
-				  cursor.blockNumber())
+		text = self.toPlainText()
+		if text.strip():
+			# Run the validator.
+			self.validateDocument.emit(self)
+		else:
+			# No text. Mark everything as ok.
+			self.setErraticLine(None)
 
-	# Validation callback.
-	# Override this in the subclass.
-	# The default implementation does nothing.
-	# In case of validation failure, call setErraticLines.
-	def validateText(self, text, currentLineNr):
-		pass
+	def __tryShowValidateErrToolTip(self, lineNr, globalPos):
+		errLineNr = self.__errLineNr
+		errLineMsg = self.__errLineMsg
+		if self.__validateEn and\
+		   not self.textCursor().hasSelection() and\
+		   errLineNr is not None and\
+		   errLineMsg:
+			errLineNr += 1
+			if lineNr in {errLineNr - 1, errLineNr, errLineNr + 1}:
+				lines = []
+				for line in errLineMsg.splitlines():
+					# Limit line length.
+					if len(line) > 237:
+						line = line[:237] + "..."
+					# Enforce line breaks.
+					while len(line) > 80:
+						lines.append(line[:80] + " \\")
+						line = line[80:]
+					lines.append(line)
+				errLineMsg = "\n".join(lines)
+				# Show the validation result.
+				QToolTip.showText(globalPos, errLineMsg,
+						  self, QRect())
+				return True
+		return False
 
-	# Mark erratic lines
-	def setErraticLines(self, errLines):
-		cursor = self.textCursor()
-		fmt = self.currentCharFormat()
-		fmt.setBackground(self.__errLineBrush)
-		extraSel = [
-			self.__makeExtraSel(self.__makeTextCursorLineSel(cursor, line),
-					    fmt)
-			for line in errLines
-		]
-		self.setExtraSelections(extraSel)
+	# Mark an erratic line
+	def setErraticLine(self, lineNr, errMsg=None):
+		if lineNr is None:
+			self.setExtraSelections(())
+		else:
+			cursor = self.textCursor()
+			fmt = self.currentCharFormat()
+			fmt.setBackground(self.__errLineBrushBg)
+			fmt.setForeground(self.__errLineBrushFg)
+			extraSel = self.__makeExtraSel(
+				self.__makeTextCursorLineSel(cursor, lineNr),
+				fmt)
+			self.setExtraSelections((extraSel,))
 
-		self.__prevErrLines = errLines
+		self.__errLineNr = lineNr
+		self.__errLineMsg = errMsg
+
+		if errMsg:
+			mouseCursorPos = QCursor.pos()
+			cursor = self.cursorForPosition(self.mapFromGlobal(mouseCursorPos))
+			self.__tryShowValidateErrToolTip(cursor.blockNumber(),
+							 mouseCursorPos)
+
+	def __toolTipEvent(self, ev):
+		cursor = self.cursorForPosition(ev.pos())
+		if not self.__tryShowValidateErrToolTip(cursor.blockNumber(),
+							ev.globalPos()):
+			QToolTip.hideText()
+
+	def event(self, ev):
+		if ev.type() == QEvent.ToolTip:
+			self.__toolTipEvent(ev)
+		return super(SourceCodeEdit, self).event(ev)
+
+	def __findText(self, replace=False):
+		dlg = self.getFindDialog()
+		if dlg:
+			dlg.hide()
+			dlg.setReplaceMode(replace)
+			dlg.show()
+
+	def findText(self):
+		self.__findText(False)
+
+	def findReplaceText(self):
+		self.__findText(True)
