@@ -2,7 +2,7 @@
 #
 # AWL simulator - GUI symbol table edit widget
 #
-# Copyright 2014 Michael Buesch <m@bues.ch>
+# Copyright 2014-2015 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
+#from awlsim.common.cython_support cimport * #@cy
 from awlsim.common.compat import *
 
 from awlsim.gui.util import *
@@ -47,21 +48,29 @@ class SymTabModel(QAbstractTableModel):
 		return self.symTab
 
 	def deleteSymbol(self, row):
-		if row >= 0 and row < len(self.symTab.symbols):
+		if row >= 0 and row < len(self.symTab):
 			self.beginResetModel()
-			del self.symTab.symbols[row]
+			del self.symTab[row]
 			self.endResetModel()
 			self.emitSourceChanged()
+			return True
+		return False
+
+	def deleteSymbols(self, rows):
+		offset = 0
+		for row in sorted(rows):
+			if self.deleteSymbol(row + offset):
+				offset -= 1
 
 	def moveSymbol(self, fromRow, toRow):
 		self.beginResetModel()
-		sym = self.symTab.symbols.pop(fromRow)
-		self.symTab.symbols.insert(toRow, sym)
+		sym = self.symTab.pop(fromRow)
+		self.symTab.insert(toRow, sym)
 		self.endResetModel()
 		self.emitSourceChanged()
 
 	def rowCount(self, parent=QModelIndex()):
-		return len(self.symTab.symbols) + 1
+		return len(self.symTab) + 1
 
 	def columnCount(self, parent=QModelIndex()):
 		return 4
@@ -71,9 +80,9 @@ class SymTabModel(QAbstractTableModel):
 			return None
 		row, column = index.row(), index.column()
 		if role in (Qt.DisplayRole, Qt.EditRole):
-			if row >= len(self.symTab.symbols):
+			if row >= len(self.symTab):
 				return None
-			sym = self.symTab.symbols[row]
+			sym = self.symTab[row]
 			if column == 0:
 				return sym.getName()
 			elif column == 1:
@@ -82,11 +91,13 @@ class SymTabModel(QAbstractTableModel):
 				return sym.getTypeString()
 			else:
 				return sym.getComment()
-		elif role == Qt.BackgroundRole:
-			if row >= len(self.symTab.symbols) or\
-			   self.symTab.symbols[row].isValid():
-				return QBrush(QColor("white"))
-			return QBrush(QColor("red"))
+		elif role in {Qt.BackgroundRole,
+			      Qt.ForegroundRole}:
+			if row < len(self.symTab) and\
+			   not self.symTab[row].isValid():
+				if role == Qt.BackgroundRole:
+					return QBrush(QColor("red"))
+				return QBrush(QColor("black"))
 		elif role in (Qt.ToolTipRole, Qt.WhatsThisRole):
 			return (
 				"The symbol name.\n(The name is case insensitive.)",
@@ -102,7 +113,7 @@ class SymTabModel(QAbstractTableModel):
 		if orientation == Qt.Horizontal:
 			return ("Symbol", "Address", "Data type", "Comment")[section]
 		else:
-			if section >= len(self.symTab.symbols):
+			if section >= len(self.symTab):
 				return "new"
 			return "%d" % (section + 1)
 
@@ -111,17 +122,29 @@ class SymTabModel(QAbstractTableModel):
 			return False
 		if role == Qt.EditRole:
 			row, column = index.row(), index.column()
-			if row >= len(self.symTab.symbols):
-				sym = Symbol()
-				self.symTab.add(sym)
-				self.rowsInserted.emit(None,
-					len(self.symTab.symbols),
-					len(self.symTab.symbols))
+			if row >= len(self.symTab):
+				nameBase = name = "__new_symbol_%d" % len(self.symTab)
+				i = 0
+				while name in self.symTab:
+					name = nameBase + "_" + str(i)
+					i += 1
+				sym = Symbol(name = name)
+				try:
+					self.symTab.add(sym)
+				except AwlSimError as e:
+					return False
+				self.rowsInserted.emit(QModelIndex(),
+					len(self.symTab),
+					len(self.symTab))
 			else:
-				sym = self.symTab.symbols[row]
+				sym = self.symTab[row]
 			try:
 				if column == 0:
-					sym.setName(value.strip())
+					if value.startswith('"') and\
+					   value.endswith('"'):
+						value = value[1:-1]
+					value = value.strip()
+					sym.setName(value)
 				elif column == 1:
 					sym.setOperatorString(value)
 				elif column == 2:
@@ -148,12 +171,11 @@ class SymTabModel(QAbstractTableModel):
 
 	def __updateSource(self):
 		try:
-			self.__source.sourceBytes = self.symTab.toASC()
+			self.symTab.toSource(self.__source)
 		except AwlSimError as e:
-			MessageBox.handleAwlSimError(None,
-				"Symbol table contains invalid characters", e)
-			return
+			return False
 		self.__needSourceUpdate = False
+		return True
 
 	def setSource(self, newSource):
 		self.beginResetModel()
@@ -161,7 +183,7 @@ class SymTabModel(QAbstractTableModel):
 			self.symTab = SymbolTable()
 			self.__source = newSource.dup()
 			self.__source.sourceBytes = b""
-			self.symTab = SymTabParser.parseData(newSource.sourceBytes)
+			self.symTab = SymTabParser.parseText(newSource.sourceText)
 		except AwlSimError as e:
 			MessageBox.handleAwlSimError(None,
 				"Could not parse symbol table information", e)
@@ -176,7 +198,10 @@ class SymTabView(QTableView):
 	def __init__(self, parent=None):
 		QTableView.__init__(self, parent)
 
-		self.verticalHeader().setMovable(True)
+		if isQt4:
+			self.verticalHeader().setMovable(True)
+		else:
+			self.verticalHeader().setSectionsMovable(True)
 		self.verticalHeader().sectionMoved.connect(self.__rowMoved)
 
 		self.pressed.connect(self.__handleMousePress)
@@ -202,12 +227,12 @@ class SymTabView(QTableView):
 		if hdr.sectionSize(3) < 200:
 			hdr.resizeSection(3, 200)
 
-	def deleteSym(self, index=None):
-		if not index:
-			index = self.currentIndex()
-		if not index:
-			return
-		self.model().deleteSymbol(index.row())
+	def deleteSyms(self, rows=None):
+		if rows is None:
+			rows = set()
+			for index in self.selectedIndexes():
+				rows.add(index.row())
+		self.model().deleteSymbols(rows)
 
 	def __handleMousePress(self, index):
 		btns = QApplication.mouseButtons()
@@ -215,10 +240,12 @@ class SymTabView(QTableView):
 			pass#TODO context menu
 
 	def keyPressEvent(self, ev):
-		QTableView.keyPressEvent(self, ev)
+		if ev.matches(QKeySequence.Delete):
+			self.deleteSyms()
+			ev.accept()
+			return
 
-		if ev.key() == Qt.Key_Delete:
-			self.deleteSym()
+		QTableView.keyPressEvent(self, ev)
 
 	def focusInEvent(self, ev):
 		QTableView.focusInEvent(self, ev)
