@@ -2,7 +2,7 @@
 #
 # AWL simulator - GUI utility functions
 #
-# Copyright 2012-2014 Michael Buesch <m@bues.ch>
+# Copyright 2012-2018 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,13 +20,15 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.common.compat import *
 
-from awlsim.common import *
-from awlsim.coreclient import *
-import awlsim.cython_helper
+from awlsim_loader.common import *
+from awlsim_loader.coreclient import *
+import awlsim_loader.cython_helper as cython_helper
+from awlsim.common.monotonic import monotonic_time #+cimport
 
 import sys
+import traceback
+import xml.sax.saxutils as saxutils
 
 if isPyPy or isJython:
 	# PySide does not work on PyPy or Jython, yet.
@@ -34,23 +36,15 @@ if isPyPy or isJython:
 	printError("Please use CPython 2.7 or CPython 3.x")
 	sys.exit(1)
 
-if awlsim.cython_helper.shouldUseCython():
+if cython_helper.shouldUseCython():
 	print("*** Using accelerated CYTHON core "
-	      "(AWLSIMCYTHON environment variable is set)")
+	      "(AWLSIM_CYTHON environment variable is set)")
 
-try:
-	from PySide.QtCore import *
-	from PySide.QtGui import *
-except ImportError as e:
-	printError("PLEASE INSTALL PySide (http://www.pyside.org/)")
-	input("Press enter to continue.")
-	sys.exit(1)
-
-import os
-import traceback
+from awlsim.gui.qt_bindings import *
 
 
-AWLSIM_HOME_URL = "http://bues.ch/h/awlsim"
+AWLSIM_HOME_DOMAIN = "awlsim.de"
+AWLSIM_HOME_URL = "https://" + AWLSIM_HOME_DOMAIN
 
 
 # Convert an integer to a dual-string
@@ -63,17 +57,23 @@ def intToDualString(value, bitWidth):
 	return ''.join(string)
 
 # Get the default fixed font
-def getDefaultFixedFont():
+def getDefaultFixedFont(pointSize=11, bold=False):
 	font = QFont()
 	font.setStyleHint(QFont.Courier)
 	font.setFamily("Courier")
-	font.setPointSize(11)
+	font.setPointSize(pointSize)
 	font.setWeight(QFont.Normal)
+	font.setBold(bold)
 	return font
 
+# Color used for errors
+def getErrorColor():
+	return QColor("#FFC0C0")
+
 def handleFatalException(parentWidget=None):
 	text = str(traceback.format_exc())
 	print("Fatal exception:\n", text)
+	text = saxutils.escape(text)
 	QMessageBox.critical(parentWidget,
 		"A fatal exception occurred",
 		"<pre>"
@@ -91,10 +91,10 @@ class MessageBox(QDialog):
 		self.setLayout(QGridLayout())
 		self.setWindowTitle(title)
 
-		self.text = "<pre>" + text + "\n</pre>"
+		self.text = "<pre>" + saxutils.escape(text) + "\n</pre>"
 		self.verboseText = None
 		if verboseText and verboseText.strip() != text.strip():
-			self.verboseText = "<pre>" + verboseText + "\n</pre>"
+			self.verboseText = "<pre>" + saxutils.escape(verboseText) + "\n</pre>"
 
 		self.textBox = QLabel(self)
 		self.textBox.setTextInteractionFlags(Qt.TextSelectableByMouse |\
@@ -132,7 +132,9 @@ class MessageBox(QDialog):
 			  text = text,
 			  verboseText = verboseText,
 			  icon = QMessageBox.Critical)
-		return dlg.exec_()
+		res = dlg.exec_()
+		dlg.deleteLater()
+		return res
 
 	@classmethod
 	def warning(cls, parent, text, verboseText=None):
@@ -141,10 +143,15 @@ class MessageBox(QDialog):
 			  text = text,
 			  verboseText = verboseText,
 			  icon = QMessageBox.Warning)
-		return dlg.exec_()
+		res = dlg.exec_()
+		dlg.deleteLater()
+		return res
 
 	@classmethod
 	def handleAwlSimError(cls, parent, description, exception):
+		if exception.getSeenByUser():
+			return cls.Accepted
+		exception.setSeenByUser()
 		def maketext(verbose):
 			text = "An exception occurred:"
 			if description:
